#------------------------------------------------------------------
# $Copyright: Copyright (c) 2013 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Header: /project/isv-agent/VProAgents/SAPComponents/SAPComponents.pm,v 1.7 2013/04/25 12:01:34 avaidya Exp $
#
# $sAuthor: $
#
# $Date: 2013/04/25 12:01:34 $
# $Revision: 1.7 $
#
# Provides:                            SAPComponents
# Author:                              rajeevk
# crAgent Build:                       1.91 2009/09/07 08:42:27
# crAgent Run Date:                    24 Aug, 2010
# Modification History:
# Known bugs/caveats:
#------------------------------------------------------------------

#------------
# Be strict..
#------------
use strict;

#----------------------
# Declare the package..
#----------------------
package SAPComponents;

use ag_i18n_inc;

#-------------------------------------
# Include required framework modules..
#-------------------------------------
use Sys;            # Support generalized UNIX commands
use Proc;           # For calls to the process routines
use Comms;          # For calls to socket(3C) routines
use Version;        # Version display generator
use VCSagentFW;     # Primary Agent Interface to the ACC Library
use VCSDefines;     # This contains constants that VCS uses

#--------------------------------
# Include standard PERL modules..
#--------------------------------
use Fcntl;
use Socket;
use File::Path qw ( mkpath );
use POSIX qw ( uname );
use File::stat;

my ( $CLUS_HOME, $sPrefix ) = ();

#------------------------
# Setup VCS/VCSone home..
#------------------------
if ( $ENV { CLUSTER_HOME } ) { # VCS 5.x/VCSOne..
  $CLUS_HOME ||= $ENV { CLUSTER_HOME } ||= "/opt/VRTSvcsone";
}
else { # VCS 4.x..
  $CLUS_HOME ||= $ENV { VCS_HOME } ||= "/opt/VRTSvcs";
}

( $sPrefix ) = $CLUS_HOME =~ /.*(vad|vcs(|one))$/;

#-----------------------------------------
# Define module-vendor specific commands..
#-----------------------------------------
my $sCmdHares  = $HARES  ||= "$CLUS_HOME/bin/hares";
my $sCmdHatype = $HATYPE ||= "$CLUS_HOME/bin/hatype";

#-----------------------------------------------------------------------
# Add our version number to the version hash and check if versions are
# requested..
#-----------------------------------------------------------------------
my $sVer = "\$Id: SAPComponents.pm,v 1.7 2013/04/25 12:01:34 avaidya Exp $::ETVHeader";
Version::TellVersion ( $sVer );

#-----------------
# Define globals..
#-----------------  
my ( $iSapStartSrv, @liRunningProcs ) = (); # Global.


#--------------------------------
# Define SAPComponents subroutines..
#--------------------------------

=pod

 -------------------------------------------------------------------------
 Function :  bool ArgsValid ( IN String  EntryPointName,
                              IN String  SAPAdmin,
                              IN String  EnvFile,
                              IN String  InstProfile,
                              IN String  ComponentType,
                              IN Integer SecondLevelMonitor,
                              IN String  MonitorProgram );

 Description : Validates the ArgList passed to the entry point.

 Parameters :

 $sEntryPointName               Entry point name
 $localSAPAdmin                 The dedicated user for the SAP Components
 $localEnvFile                  Absolute path to environment file to source
 $localInstProfile              Specifies the full path and file name of the SAP Instance Profile.
 $localComponentType            String identifier that classifies and describes the SAP Component.
 $localSecondLevelMonitor       Cycle count to run in-depth monitoring
 $localMonitorProgram           Full path to external monitor program
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub ArgsValid {
  #----------------------------------------
  # Set up local vars from passed in ones..
  #----------------------------------------
  my ( $sEntryPointName,
       $localSAPAdmin,
       $localEnvFile,
       $localInstProfile,
       $localComponentType,
       $localSecondLevelMonitor,
       $localMonitorProgram ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20001, "Subroutine <%s> called with:
  EntryPointName           [%s]
  SAPAdmin                 [%s]
  EnvFile                  [%s]
  InstProfile              [%s]
  ComponentType            [%s]
  SecondLevelMonitor       [%s]
  MonitorProgram           [%s]",

  $sMyName,   $sEntryPointName,   $localSAPAdmin,    $localEnvFile,
  $localInstProfile, $localComponentType, $localSecondLevelMonitor,
  $localMonitorProgram );

  #-------------------------------------
  # Set return code to hopeful success..
  #-------------------------------------
  my $iReturnCode = 1;

  #--------------------------------------------
  # Create hash to weed out empty args values..
  #--------------------------------------------
  my ( @lsMissing, %hsArgList ) = ();
  my @lsArgs =
  qw ( EntryPointName   SAPAdmin   EnvFile   InstProfile   ComponentType
       SecondLevelMonitor MonitorProgram );

  @hsArgList { @lsArgs } = @_; # Assign..

  #----------------------------------------
  # Reqd. attributes based on entry point..
  #----------------------------------------
  my @lsOnlineArgs  = qw ( EnvFile SAPAdmin InstProfile ComponentType );
  my @lsOfflineArgs = qw ( SAPAdmin InstProfile ComponentType );
  my @lsMonitorArgs = qw ( SAPAdmin InstProfile ComponentType );
  my @lsCleanArgs   = qw ( SAPAdmin InstProfile );

  my %hlsNeeded = ( 'online'  => \@lsOnlineArgs,
                    'offline' => \@lsOfflineArgs,
                    'monitor' => \@lsMonitorArgs,
                    'clean'   => \@lsCleanArgs );

  my @lsReqdArgs = @{ $hlsNeeded { $sEntryPointName } };

  #-----------------------
  # Check what's missing..
  #-----------------------
  if ( @lsMissing = grep { $hsArgList { $_ } !~ /\S/o } @lsReqdArgs ) {
    &logError ( 20002,
    "<%s> The following mandatory fields were not set [%s], Total [%s]",
    $sMyName, join ( ' ', @lsMissing ), scalar @lsMissing );
    $iReturnCode = 0; # Failure..
  }

  $iReturnCode || return $iReturnCode; # Return, if failed..

  #-----------------------------
  # Remove SLM file, if exists..
  #-----------------------------
  if ( $sEntryPointName ne 'monitor' ) {
    VCSagentFW::RunSecondLevelMonitor ( 1 ); # Delete..
  }

  #-------------------------------
  # Individual validation starts..
  #-------------------------------
  
  my ($sSAPSID, $sInstName, $sSAPHost) = ();
  my (@lsWords) = ();

  (@lsWords) = split( /\//, $localInstProfile);
  ($sSAPSID, $sInstName, $sSAPHost) = split(/_/, $lsWords[-1]);

  #-------------------------------------------------------
  # Validation for online offline monitor entry point(s)..
  #-------------------------------------------------------
  if ( $sEntryPointName =~ /^(online|offline|monitor)$/o ) {
    #------------------------------
    # Validate parameter SAPAdmin..
    #------------------------------
    if ( ! defined getpwnam $localSAPAdmin ) {
      &logError ( 20003, "<%s> SAPAdmin [%s] is not a valid UNIX login",
      $sMyName, $localSAPAdmin );
      $iReturnCode = 0; # Failure..
    }
  }
  
  #---------------------------------------
  # Validation for online entry point(s)..
  #---------------------------------------
  if ( $sEntryPointName eq "online" ) {
    #-----------------------------
    # Validate parameter EnvFile..
    #-----------------------------
    if ( ! -e $localEnvFile ) {
      &logError ( 20004, "<%s> EnvFile [%s] file does not exist",
      $sMyName, $localEnvFile );
      $iReturnCode = 0; # Failure..
    }

    #---------------------------------------
    # Validate SAPHost..
    #---------------------------------------
    if ( ! inet_aton ( $sSAPHost ) ) {
      &logError ( 20005,
      "<%s> InstProfile [%s] not terminated with valid hostname [%s]",
      $sMyName, $localInstProfile, $sSAPHost );
      $iReturnCode = 0; # Failure..
    }

    #------------------------------------
    # Validate parameter MonitorProgram..
    #------------------------------------
    if ( $localMonitorProgram =~ /(\S+)/ && # Defined..
         ! -x $1 ) {
      &logError ( 20006, "<%s> MonitorProgram [%s] is not executable",
      $sMyName, $1 );
      $iReturnCode = 0; # Failure..
    }
    #-------------------------------
    # Validate parameter InstProfile..
    #-------------------------------
    if ( ! -e $localInstProfile ) { # move it to only online
      &logError ( 20007,
      "<%s> InstProfile [%s] directory does not exist",
      $sMyName, $localInstProfile );
      $iReturnCode = 0; # Failure..
    }
  }
  
  #----------------------------------
  # Validation for all entry points..
  #----------------------------------
  #-----------------------------------------
  # SAPSID.. must be three characters long..
  #-----------------------------------------
  if ( $sSAPSID !~ /^\S{3}$/o ) {
    &logError ( 20008,
    "<%s> SAPSID [%s] has to be exactly 3 non-white alpha-numeric characters long",
    $sMyName, $sSAPSID );
    $iReturnCode = 0; # Failure..
  }

  #-----------------------------------
  # InstName. Last two chars 00 - 99..
  #-----------------------------------
  unless ( $sInstName =~ /(\d{2})$/o ) {
    &logError ( 20009,
    "<%s> Last two characters of InstName [%s] not between 00-99",
    $sMyName, $sInstName );
    $iReturnCode = 0; # Failure..
  }

  #-----------------------
  # Valid ComponentTypes..
  #-----------------------
  my %hsInstTypes = ( 'ContentServerWD' => 1,
					  'Gateway' => 1,
                      'WebDispatcher' => 1,
                      'MDM' => 1  );
  if ( ! $hsInstTypes { $localComponentType } ) {
    &logError ( 20010,
    "<%s> InstType [%s] invalid. Has to be one of [%s]",
    $sMyName, $localComponentType, join ( ' ', keys %hsInstTypes ) );
    $iReturnCode = 0; # Failure..
  }
  $iReturnCode ||
  &logWarn ( 20011,
  "<%s> Some of the attributes could not be validated. Returning [%s]",
  $sMyName, $iReturnCode );

  return $iReturnCode ;
}

=pod

 -------------------------------------------------------------------------
 Function : bool Start ( IN String ComponentType,
                         IN String EnvFile,
                         IN String InstProfile,
                         IN String SAPAdmin,
                         IN String SAPSID,
                         IN String SAPHome,
                         IN String InstName,
                         IN String SAPHost
                         IN Integer SecondLevelMonitor 
                         IN String  MonitorProgram);

 Description : Start logic needed for online entry pt

 Parameters :

 $sComponentType            String identifier that classifies and describes the SAP Component.
 $sEnvFile                  Absolute path to environment file to source
 $sInstProfile              Specifies the full path and file name of the SAP Instance Profile.
 $sSAPAdmin                 The dedicated user for the SAP Components
 $sSAPSID                   SAP System ID
 $sSAPHome                  SAP Instance Home
 $sInstName                 SAP Instance Name
 $sSAPHost                  SAP Instance virtual host name
 $iSecondLevelMonitor       Cycle count to run in-depth monitoring
 $sMonitorProgram            External Monitor command
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub Start {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sComponentType,
       $sEnvFile,
       $sInstProfile,
       $sSAPAdmin,
       $sSAPSID,
       $sSAPHome,
       $sInstName,
       $sSAPHost,
       $iSecondLevelMonitor,
       $sMonitorProgram ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20012, "Subroutine <%s> called with:
  ComponentType            [%s]
  EnvFile                  [%s]
  InstProfile              [%s]
  SAPAdmin                 [%s]
  SAPSID                   [%s]
  SAPHome                  [%s]
  InstName                 [%s]
  SAPHost                  [%s]
  SecondLevelMonitor       [%s]
  MonitorProgram           [%s]",

  $sMyName, $sComponentType, $sEnvFile, $sInstProfile,  $sSAPAdmin,
  $sSAPSID, $sSAPHome, $sInstName, $sSAPHost, $iSecondLevelMonitor, $sMonitorProgram );

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1;

  #-------------------------
  # Define local variables..
  #-------------------------
  my ( $sCommand, $sCommandArgs, $iTillWhen, $iExitCode, $iTimeout,
       @lsCmdout, @liPids ) = ();

  #------------------------------------------------
  # Whack any preexisting SAPComponents processes..
  #------------------------------------------------
  &logTrace ( 20013,
  "<%s> Killing SAPComponents processes for this instance", $sMyName );

  if ( SAPComponents::KillSAPComponentsProcesses ( $sInstProfile,
                                                   $sComponentType,
                                                   $sSAPAdmin ) ) {
    &logTrace ( 20014,
    "<%s> Processes for this instance do not exist before start-up",
    $sMyName );
  }
  else {
    &logError ( 20015,
    "<%s> Could not remove all processes for this instance. Exiting",
    $sMyName );
    $iReturnCode = 0; # Failure..
  }
  
  #--------------------
  # Return, if failed..
  #--------------------
  $iReturnCode || return $iReturnCode;

  #---------------------------------------------------------------------
  # Upper bound.. Time in EPOCH when entry pt started + VCS time limit..
  #---------------------------------------------------------------------
  $iTillWhen = $^T + SAPComponents::GetEngineTimeout ( 1 );

  my $iInstID = ();
  #--------------------------------------------------------
  # Clean IPC resources for the instance ..
  #--------------------------------------------------------
  ($iInstID ) = $sInstName =~ /(\d{2})$/o;

  my $bSkipCleanipc = 0;
  $sCommand = "$sSAPHome/cleanipc";
  if (! -e $sCommand) {
    &logInfo ( 20185, "<%s> Binary file [%s] does not exist.", $sMyName, $sCommand);
    if ( $sComponentType =~ /^(ContentServerWD|Gateway)$/o ) {
      &logInfo ( 20186, "<%s> The agent will skip executing the command [%s] for Component [%s].", $sMyName, $sCommand, $sComponentType);
      $bSkipCleanipc = 1;
    }
    else {
      &logError ( 20187, "<%s> Binary file [%s] does not exist for Component [%s]. Returning failure.", $sMyName, $sCommand, $sComponentType);
      $iReturnCode = 0; # Failure.. 
    }
  #--------------------
  # Return, if failed..
  #--------------------
  $iReturnCode || return $iReturnCode;
  }
  if ( !$bSkipCleanipc) { 
    $sCommandArgs = "$iInstID remove";
    ( @lsCmdout, $iExitCode, $iTimeout ) = ();
  
    &logTrace ( 20016,
    "<%s> Cleaning IPC resources for SAP instance with Cmd [%s] Args [%s]",
    $sMyName, $sCommand, $sCommandArgs );
  
    if ( ( $iTimeout = $iTillWhen - time ) <= 10 ) { # Get time bearings..
      &logError ( 20017,
      "<%s> Insufficient time to run cleanipc. Short by [%s] sec",
      $sMyName, abs ( $iTimeout ) );
  
      $iReturnCode = 0; # Failure..
    }
    elsif ( $iReturnCode =
      SAPComponents::ExecLocalCmd ( $sSAPAdmin,
                                    $sEnvFile,
                                    $sCommand,
                                    $sCommandArgs,
                                    \$iExitCode,
                                    $iTimeout,
                                    \@lsCmdout ) ) {
      &logTrace ( 20018,
      "<%s> Cmd [%s] with args [%s] executed successfully",
      $sMyName, $sCommand, $sCommandArgs );
    }
   
    #--------------------
    # Return, if failed..
    #--------------------
    $iReturnCode || return $iReturnCode;
  }
  my ($sKillFile, $sShutdownFile, @lsFiles ) = ();

  #------------------
  # Remove kill.sap..
  #------------------
  $sKillFile = "/usr/sap/$sSAPSID/$sInstName/work/kill.sap";

  if ( -e $sKillFile ) {
    &logTrace ( 20019,
    "<%s> Located file [%s] for removal", $sMyName, $sKillFile );
    push ( @lsFiles, $sKillFile );
  }

  #----------------------
  # Remove shutdown.sap..
  #----------------------
  $sShutdownFile = "/usr/sap/$sSAPSID/$sInstName/work/shutdown.sap";

  if ( -e $sShutdownFile ) {
    &logTrace ( 20020,
    "<%s> Located file [%s] for removal", $sMyName, $sShutdownFile );
    push ( @lsFiles, $sShutdownFile );
  }
 
  #-------------------
  # Remove the files..
  #-------------------
  if ( scalar @lsFiles && ! Sys::UnlinkFilesAsUser ( @lsFiles ) ) {
    &logError ( 20021, "<%s> Failed to remove stale file(s) [%s]",
    $sMyName, join ( ' ', @lsFiles ) );
  }
  
  #-------------------------------------------
  # Get re-direct string based on user shell..
  #-------------------------------------------
  my $sRdStr = ( getpwnam $sSAPAdmin ) [-1] =~ /csh$/o ? ">>& /dev/null":
                                                         "1>> /dev/null 2>&1";
  #------------------------------------------
  # Define start command and args variables..
  #------------------------------------------

  ( @lsCmdout, $iExitCode, $iTimeout ) = ();
  $sCommand = "$sSAPHome/sapcontrol";
  if ( -e $sCommand ) {

    $sCommandArgs = "-nr $iInstID -function StartService $sSAPSID";

    &logTrace ( 20334,
    "<%s>Starting SAPComponents instance with Cmd [%s] Args [%s]",
    $sMyName, $sCommand, $sCommandArgs );

    #------------------
    # Request startup..
    #------------------
    if ( ( $iTimeout = $iTillWhen - time ) <= 10 ) { # Get time bearings..
      &logError ( 20335,
      "<%s>Insufficient time to continue boot up. Short by [%s] sec",
      $sMyName, abs ( $iTimeout ) );

      $iReturnCode = 0; # Failure..
    }
    elsif ( $iReturnCode =
      SAPComponents::ExecLocalCmd ( $sSAPAdmin,
                                  $sEnvFile,
                                  $sCommand,
                                  $sCommandArgs,
                                  \$iExitCode,
                                  $iTimeout,
                                  \@lsCmdout ) ) {
      &logTrace ( 20336,
      "<%s>Cmd [%s] with args [%s] executed successfully",
      $sMyName, $sCommand, $sCommandArgs );
    }

    #-------------------
    # Return if failed..
    #-------------------

    $iReturnCode || return $iReturnCode;
    #--------------------
    # Delay entry point..
    #--------------------
    my $iSleepTime = 10; # Secs..
    sleep $iSleepTime; # Sec..

    $sCommand = "$sSAPHome/sapcontrol";
    $sCommandArgs = "-nr $iInstID -function Start";
  }
  else
  {
    $sCommand = "$sSAPHome/startsap";
    $sCommandArgs = "r3 $sInstName $sSAPHost $sRdStr";
  }

  &logTrace ( 20022,
  "<%s> Starting SAPComponents instance with Cmd [%s] Args [%s]",
  $sMyName, $sCommand, $sCommandArgs );

  #------------------
  # Request startup..
  #------------------
  if ( ( $iTimeout = $iTillWhen - time ) <= 10 ) { # Get time bearings..
    &logError ( 20023,
    "<%s> Insufficient time to continue boot up. Short by [%s] sec",
    $sMyName, abs ( $iTimeout ) );

    $iReturnCode = 0; # Failure..
  }
  elsif ( $iReturnCode =
    SAPComponents::ExecLocalCmd ( $sSAPAdmin, 
                                  $sEnvFile, 
                                  $sCommand, 
                                  $sCommandArgs, 
                                  \$iExitCode, 
                                  $iTimeout, 
                                  \@lsCmdout ) ) {
    &logTrace ( 20024,
    "<%s> Cmd [%s] with args [%s] executed successfully",
    $sMyName, $sCommand, $sCommandArgs );
  }

  #-------------------
  # Return if failed..
  #-------------------
  $iReturnCode || return $iReturnCode;


  #--------------------
  # Delay entry point..
  #--------------------
  my $iSleepTime = 10; # Secs..
  
  while ( ( $iTimeout = $iTillWhen - time ) > 10 ) {
    @liPids = (); # Initialise..

    sleep $iSleepTime; # Sec..

    &logTrace ( 20025,
    "<%s> Confirming start-up with [%s] sec in hand",
    $sMyName, $iTimeout - $iSleepTime );

    #---------
    # Do FLM..
    #---------
    if ( VCSDef_MonitorExit::ONLINE ==
         SAPComponents::GetProcState ( \@liPids,
                                       $sComponentType,
                                       $sSAPAdmin,
                                       $sInstProfile,
                                       1 ) ) {          # Running FFLM..
      #---------------------
      # Start-up confirmed..
      #---------------------
      &logInfo ( 20026,
      "<%s> Process check confirmed start-up of this instance of SAP Component",
      $sMyName );

      $iReturnCode = 1; # Success..
      last;
    }
    else {
      &logTrace ( 20027,
      "<%s> Process check could not confirm start-up. Sleeping for [%s] sec",
      $sMyName, $iSleepTime );

      $iReturnCode = 0; # Failure..
    }
  }
  #---------------------------------------------
  # Delay Entry point for SeconfLevel monitoring
  #---------------------------------------------

  if ( $iSecondLevelMonitor ) {      # If enabled..
    while ( ( $iTimeout = $iTillWhen - time ) > 10 ) {
      @liPids = (); # Initialise..

      sleep $iSleepTime; # Sec..

      &logTrace ( 20028,
      "<%s> Confirming start-up with [%s] sec in hand by health check",
      $sMyName, $iTimeout - $iSleepTime );

      #---------
      # Do SLM..
      #---------
      if ( VCSDef_MonitorExit::ONLINE ==                      # Online after..
           SAPComponents::SecondLevelMonitor ( $sComponentType, # Running SLM..
                                               $sEnvFile,
                                               $sInstProfile,
                                               $sSAPAdmin,
                                               $sSAPHome, 
                                               $sInstName, 
                                               $sSAPHost,
                                               $iTillWhen ) ) { 
        #---------------------
        # Start-up confirmed..
        #---------------------
        &logInfo ( 20029,
        "<%s> Second Level Monitor confirmed start-up of this instance of SAP Component",
        $sMyName );

        $iReturnCode = 1; # Success..
        last;
      }
      else {
        &logTrace ( 20030,
        "<%s> Second Level Monitor could not confirm start-up. Sleeping for [%s] sec",
        $sMyName, $iSleepTime );

        $iReturnCode = 0; # Failure..
      }
    }
  }
  
  #----------------------------------------------------------
  # MonitorProgram defined..
  # Delay Entry proint to confirm startup by external monitor
  #----------------------------------------------------------
  if ( $sMonitorProgram =~ /\S+/ ) {
    my ( $sExtCmd, $sExtCmdArgs ) = $sMonitorProgram # Separate
                                  =~ /(\S+)\s*(.*)/; # Cmd and args..

    $sExtCmdArgs = '' unless defined $sExtCmdArgs;   # Prevent undefs..
  
    if ( $sExtCmd ) {                                # Defined..
      while ( ( $iTimeout = $iTillWhen - time ) > 10 ) {
        if ( VCSDef_MonitorExit::ONLINE ==             # and found online..
             SAPComponents::ExternalMonitor ( $sSAPAdmin,
                                              $sEnvFile,
                                              $sExtCmd,
                                              $sExtCmdArgs,
                                              $iTillWhen ) ) {
          &logTrace ( 20031,
          "<%s> Confirmed start-up of SAP component after external monitor [%s] 
          with args [%s]",
          $sMyName, $sExtCmd, $sExtCmdArgs );

          $iReturnCode = 1; # Success..
          last;
        }
        else {
          &logError ( 20032,
          "<%s> Could not confirm start-up after  
          executing external monitor [%s] with args [%s]. Sleeping for [%s] sec",
          $sMyName, $sExtCmd, $sExtCmdArgs, $iSleepTime );

          sleep $iSleepTime; # Sec..
          $iReturnCode = 0;  # Failure..
        }
      }
    }
  }

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : bool Stop ( IN String ComponentType,
                        IN String EnvFile,
                        IN String InstProfile,
                        IN String SAPAdmin, 
                        IN String SAPSID,
                        IN String SAPHome,
                        IN String InstName 
                        IN String SAPHost );

 Description : Stop logic needed for offline entry pt

 Parameters :

 $sComponentType            String identifier that classifies and describes the SAP Component.
 $sEnvFile                  Absolute path to environment file to source
 $sInstProfile              Specifies the full path and file name of the SAP Instance Profile.
 $sSAPAdmin                 The dedicated user for the SAP Components
 $sSAPSID                   SAP System ID
 $sSAPHome                  SAP Instance Home
 $sInstName                 SAP Instance Name
 $sSAPHost                  SAP Instance virtual host name
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub Stop {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sComponentType,
       $sEnvFile,
       $sInstProfile,
       $sSAPAdmin,
       $sSAPSID, 
       $sSAPHome, 
       $sInstName,
       $sSAPHost ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20033, "Subroutine <%s> called with:
  ComponentType            [%s]
  EnvFile                  [%s]
  InstProfile              [%s]
  SAPAdmin                 [%s]
  SAPSID                   [%s]
  SAPHome                  [%s]
  InstName                 [%s]
  SAPHost                  [%s]",

  $sMyName, $sComponentType, $sEnvFile, $sInstProfile, $sSAPAdmin,
  $sSAPSID, $sSAPHome, $sInstName, $sSAPHost );

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1; # Success..

  #---------------------------------------------------------------------
  # Upper bound.. Time in EPOCH when entry pt started + VCS time limit..
  #---------------------------------------------------------------------
  my $iTillWhen = $^T + SAPComponents::GetEngineTimeout ( 1 ); # With margin..

  #------------------------------------------
  # Declare stop command and args variables..
  #------------------------------------------
  my ( $sCommand, $sCommandArgs, $iTimeout, $iExitCode, $iSleepTime,
       @lsCmdout, @liPids ) = ();
 
  #-------------------------------------------
  # Get re-direct string based on user shell..
  #-------------------------------------------
  my $sRdStr = ( getpwnam $sSAPAdmin ) [-1] =~ /csh$/o ? ">>& /dev/null":
                                                         "1>> /dev/null 2>&1"; 
  #------------------------------------
  # Define stop command and arguments..
  #------------------------------------

  ( $iExitCode, $iTimeout, @lsCmdout ) = (); # Truncate.. 
  
    $sCommand = "$sSAPHome/sapcontrol";
    if ( -e $sCommand ) {
    my $iInstID = (); 
    ($iInstID ) = $sInstName =~ /(\d{2})$/o;

    $sCommandArgs = "-nr $iInstID -function Stop";

    &logTrace ( 20337,
    "<%s>Stopping SAPComponents with Command [%s] Args [%s]",
    $sMyName, $sCommand, $sCommandArgs );

    #------------------------
    # Request shutdown..
    #------------------------
    if ( ( $iTimeout = $iTillWhen - time ) <= 10 ) { # Get time bearings..
      &logError ( 20338,
      "<%s>Insufficient time to continue shutdown. Short by [%s] sec",
      $sMyName, abs ( $iTimeout ) );

      $iReturnCode = 0; # Failure..
    }
    elsif ( $iReturnCode =
            SAPComponents::ExecLocalCmd ( $sSAPAdmin,
                                          $sEnvFile,
                                          $sCommand,
                                          $sCommandArgs,
                                          \$iExitCode,
                                          $iTimeout,
                                          \@lsCmdout ) ) {
      &logTrace ( 20339,
      "<%s>Cmd [%s] with args [%s] executed successfully",
      $sMyName, $sCommand, $sCommandArgs );
    }

    #--------------------
    # Delay entry point..
    #--------------------
    my $iSleepTime = 10; # Secs..
    sleep $iSleepTime; # Sec..

    $sCommand = "$sSAPHome/sapcontrol";
    $sCommandArgs = "-nr $iInstID -function StopService $sSAPSID";
  }
  else
  {
    $sCommand = "$sSAPHome/stopsap";
    $sCommandArgs = "r3 $sInstName $sSAPHost $sRdStr";
  }

  &logTrace ( 20034,
  "<%s> Stopping SAPComponents with Command [%s] Args [%s]",
  $sMyName, $sCommand, $sCommandArgs );

  #------------------------
  # Request shutdown..
  #------------------------
  if ( ( $iTimeout = $iTillWhen - time ) <= 10 ) { # Get time bearings..
    &logError ( 20035,
    "<%s> Insufficient time to continue shutdown. Short by [%s] sec",
    $sMyName, abs ( $iTimeout ) );

    $iReturnCode = 0; # Failure..
  }
  elsif ( $iReturnCode =
          SAPComponents::ExecLocalCmd ( $sSAPAdmin, 
                                        $sEnvFile, 
                                        $sCommand, 
                                        $sCommandArgs, 
                                        \$iExitCode, 
                                        $iTimeout, 
                                        \@lsCmdout ) ) {
    &logTrace ( 20036,
    "<%s> Cmd [%s] with args [%s] executed successfully",
    $sMyName, $sCommand, $sCommandArgs );
  }


  #--------------------
  # Delay entry point..
  #--------------------
  if ( $iReturnCode ) { # Success..
    #-----------------
    # Set sleep time..
    #-----------------
    $iSleepTime = 10; # Sec..
  
    &logInfo ( 20037,
    "<%s> Delaying exit to confirm shutdown..", $sMyName );

    #--------------------
    # Delay entry point..
    #--------------------
    while ( ( $iTimeout = $iTillWhen - time ) > 20 ) {
      #-----------------------
      # Wait before retrying..
      #-----------------------
      sleep $iSleepTime; # Secs..
  
      SAPComponents::GetProcState ( \@liPids,
                                    $sComponentType,
                                    $sSAPAdmin,
                                    $sInstProfile,
                                    0 ); # Request regular..
  
      #------------------------------
      # Check for pending processes..
      #------------------------------
      if ( scalar @liPids ) {
     
        #-----------------------------------------------
        # Check if the last process is not sapstartsrv..
        #-----------------------------------------------
        if ( 1 == scalar @liPids   && # Last remaining..
             defined $iSapStartSrv && #
             $iSapStartSrv == $liPids [0] ) {
          last;
        }
  
        &logTrace ( 20038,
        "<%s> Could not confirm shutdown. Retrying in [%s] sec.
        Delaying kill with [%s] sec in hand",
        $sMyName, $iSleepTime, $iTimeout - $iSleepTime );
      }
      else {
        &logInfo ( 20039,
        "<%s> Confirmed shutdown of SAPComponents instance",
        $sMyName );
        last; # Get out..
      }
    }
  }

  #-----------------------------------------------
  # Whack any persisting SAPComponents processes..
  #-----------------------------------------------
  &logTrace ( 20040,
  "<%s> Killing SAPComponents processes for this instance", $sMyName );

  if ( SAPComponents::KillSAPComponentsProcesses ( $sInstProfile,
                                                   $sComponentType,
                                                   $sSAPAdmin ) ) {
    &logTrace ( 20041,
    "<%s> Processes for this instance do not exist after shutdown",
    $sMyName );
  }
  else {
    &logError ( 20042,
    "<%s> Could not remove all processes for this instance. Exiting",
    $sMyName );

    $iReturnCode = 0; # Failure..
  }
  
  my ($sKillFile, $sShutdownFile, @lsFiles ) = ();

  #------------------
  # Remove kill.sap..
  #------------------
  $sKillFile = "/usr/sap/$sSAPSID/$sInstName/work/kill.sap";

  if ( -e $sKillFile ) {
    &logTrace ( 20043,
    "<%s> Located file [%s] for removal", $sMyName, $sKillFile );
    push ( @lsFiles, $sKillFile );
  }

  #----------------------
  # Remove shutdown.sap..
  #----------------------
  $sShutdownFile = "/usr/sap/$sSAPSID/$sInstName/work/shutdown.sap";

  if ( -e $sShutdownFile ) {
    &logTrace ( 20044,
    "<%s> Located file [%s] for removal", $sMyName, $sShutdownFile );
    push ( @lsFiles, $sShutdownFile );
  }
  
  #-------------------
  # Remove the files..
  #-------------------
  if ( scalar @lsFiles && ! Sys::UnlinkFilesAsUser ( @lsFiles ) ) {
    &logError ( 20045, "<%s> Failed to remove stale file(s) [%s]",
    $sMyName, join ( ' ', @lsFiles ) );
    $iReturnCode = 0; # Failure..
  }

  #-----------------------------------
  # Remove preexisting IPC resources..
  #-----------------------------------
  #----------------------------------------
  # Define cleanipc command and arguments..
  #----------------------------------------
  my $iInstID = ();
  ($iInstID ) = $sInstName =~ /(\d{2})$/o;
  my $bSkipCleanipc = 0;
  $sCommand = "$sSAPHome/cleanipc";
  if (! -e $sCommand) {
    &logInfo ( 20188, "<%s> Binary file [%s] does not exist.", $sMyName, $sCommand);
    if ( $sComponentType =~ /^(ContentServerWD|Gateway)$/o ) {
      &logInfo ( 20189, "<%s> The agent will skip executing the command [%s] for Component [%s].", $sMyName, $sCommand, $sComponentType);
      $bSkipCleanipc = 1;
    }
    else {
      &logError ( 20190, "<%s> Binary file [%s] does not exist for Component [%s]. Returning failure.", $sMyName, $sCommand, $sComponentType);
      $iReturnCode = 0; # Failure.. 
    }
  #--------------------
  # Return, if failed..
  #--------------------
  $iReturnCode || return $iReturnCode;
  }
  if ( !$bSkipCleanipc) { 
    $sCommandArgs = "$iInstID remove";
    ( $iExitCode, $iTimeout, @lsCmdout ) = (); # Truncate..
  
    &logTrace ( 20046,
    "<%s> Stopping SAPComponent with Command [%s] Args [%s]",
    $sMyName, $sCommand, $sCommandArgs );
  
    #------------------------
    # Request shutdown..
    #------------------------
    if ( ( $iTimeout = $iTillWhen - time ) <= 10 ) { # Get time bearings..
      &logError ( 20047,
      "<%s> Insufficient time to continue shutdown. Short by [%s] sec",
      $sMyName, abs ( $iTimeout ) );
  
      $iReturnCode = 0; # Failure..
    }
    elsif ( $iReturnCode =
            SAPComponents::ExecLocalCmd ( $sSAPAdmin,
                                          $sEnvFile,
                                          $sCommand,
                                          $sCommandArgs,
                                          \$iExitCode,
                                          $iTimeout,
                                          \@lsCmdout ) ) {
      &logTrace ( 20048,
      "<%s> Cmd [%s] with args [%s] executed successfully",
      $sMyName, $sCommand, $sCommandArgs );
    }
    else {
      &logError ( 20049,
      "<%s> Could not remove all IPC resources for User [%s]",
      $sMyName, $sSAPAdmin );
      $iReturnCode = 0; # Failure..
    }
  }
  my $sResourceName = $ENV { VCS_LOG_RESOURCE_NAME };
  SAPComponents::DeletePidFile ( $sResourceName );

  return $iReturnCode;
}

=pod

 -------------------------------------------------------------------------
 Function : bool Kill ( IN Integer CleanReason,
                        IN String ComponentType,
                        IN String EnvFile,
                        IN String InstProfile,
                        IN String SAPAdmin,
                        IN String SAPSID,
                        IN String SAPHome,
                        IN String InstName 
                        IN String SAPHost );

 Description : Forces shutdown of resource needed for clean entry point

 Parameters :

 $iCleanReason              Why clean was called
 $sComponentType            String identifier that classifies and describes the SAP Component.
 $sEnvFile                  Absolute path to environment file to source
 $sInstProfile              Specifies the full path and file name of the SAP Instance Profile.
 $sSAPAdmin                 The dedicated user for the SAP Components
 $sSAPSID                   SAP System ID
 $sSAPHome                  SAP Instance Home
 $sInstName                 SAP Instance Name
 $sSAPHost                  SAP Instance virtual host name

 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub Kill {
  #----------------
  # Get Arguments..
  #----------------
  my ( $iCleanReason,
       $sComponentType,
       $sEnvFile,
       $sInstProfile,
       $sSAPAdmin,
       $sSAPSID,
       $sSAPHome,
       $sInstName,
       $sSAPHost ) = @_;

  #-----------------------------
  # Figure out why were called..
  #-----------------------------
  my ( $sCleanReason, %hsCleanReasons ) = ();
  my @lsCleanStrings = qw ( Offline_timed_out    Offline_was_ineffective
                            Online_timed_out      Online_was_ineffective
                            Found_offline_unexpectedly Monitor_timed_out );

  @hsCleanReasons { 0 .. $#lsCleanStrings } = @lsCleanStrings; # Populate..
  ( $sCleanReason = $hsCleanReasons { $iCleanReason } ||       # Reason..
                    $iCleanReason ) =~ s/_/ /g;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20050, "Subroutine <%s> called with:
  CleanReason              [%s]
  ComponentType            [%s]
  EnvFile                  [%s]
  InstProfile              [%s]
  SAPAdmin                 [%s]
  SAPSID                   [%s]
  SAPHome                  [%s]
  InstName                 [%s]
  SAPHost                  [%s]",

  $sMyName, $sCleanReason, $sComponentType, $sEnvFile, $sInstProfile, 
  $sSAPAdmin, $sSAPSID, $sSAPHome, $sInstName, $sSAPHost );

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1;

  #---------------------------------------------------------------------
  # Upper bound.. Time in EPOCH when entry pt started + VCS time limit..
  #---------------------------------------------------------------------
  my $iTillWhen = $^T + SAPComponents::GetEngineTimeout ( 1 ); # With margin..

  #--------------------------
  # Declare local variables..
  #--------------------------
  my ( $sCommand, $sCommandArgs, $iTimeout, $iExitCode,
       @lsFiles, @lsCmdout ) = ();
   
  #-------------------------------------------
  # Get re-direct string based on user shell..
  #-------------------------------------------
  my $sRdStr = ( getpwnam $sSAPAdmin ) [-1] =~ /csh$/o ? ">>& /dev/null":
                                                         "1>> /dev/null 2>&1";
  #------------------------------------
  # Define clean command and arguments..
  #------------------------------------
  if( $sComponentType !~ /^(MDM|ContentServerWD)$/o ) { 
    unless ( $hsCleanReasons { $iCleanReason } eq 'Offline_timed_out' ||
           $hsCleanReasons { $iCleanReason } eq 'Offline_was_ineffective' ) {
    $sCommand = "$sSAPHome/sapcontrol";
    if ( -e $sCommand ) {
      my $iInstID = (); 
      ($iInstID ) = $sInstName =~ /(\d{2})$/o;

      $sCommandArgs = "-nr $iInstID -function Stop";

      &logTrace ( 20194,
      "<%s>Stopping SAPComponents with Command [%s] Args [%s]",
      $sMyName, $sCommand, $sCommandArgs );

      #------------------------
      # Request shutdown..
      #------------------------
      if ( ( $iTimeout = $iTillWhen - time ) <= 10 ) { # Get time bearings..
        &logError ( 20195,
        "<%s>Insufficient time to continue shutdown. Short by [%s] sec",
        $sMyName, abs ( $iTimeout ) );

        $iReturnCode = 0; # Failure..
      }
      elsif ( $iReturnCode =
              SAPComponents::ExecLocalCmd ( $sSAPAdmin,
                                            $sEnvFile,
                                            $sCommand,
                                            $sCommandArgs,
                                            \$iExitCode,
                                            $iTimeout,
                                            \@lsCmdout ) ) {
        &logTrace ( 20196,
        "<%s>Cmd [%s] with args [%s] executed successfully",
        $sMyName, $sCommand, $sCommandArgs );
      }

      #--------------------
      # Delay entry point..
      #--------------------
      my $iSleepTime = 10; # Secs..
      sleep $iSleepTime; # Sec..

      $sCommand = "$sSAPHome/sapcontrol";
      $sCommandArgs = "-nr $iInstID -function StopService $sSAPSID";
    } else {
      $sCommand = "$sSAPHome/stopsap";
      $sCommandArgs = "r3 $sInstName $sSAPHost $sRdStr";
    }
    ( $iExitCode, $iTimeout, @lsCmdout ) = (); # Truncate.. 

      #-------------------
      # Try to be gentle..
      #-------------------
      &logInfo ( 20340,
      "<%s> Attempting to stop SAPComponents with Command [%s] Args [%s]",
      $sMyName, $sCommand, $sCommandArgs );
  
      #--------------------
      # Request shutdown..
      #--------------------
      if ( ( $iTimeout = $iTillWhen - time ) <= 10 ) { # Get time bearings..
        &logError ( 20341,
        "<%s> Insufficient time to continue shutdown. Short by [%s] sec",
        $sMyName, abs ( $iTimeout ) );

        $iReturnCode = 0; # Failure..
      }
      elsif ( SAPComponents::ExecLocalCmd ( $sSAPAdmin, 
                                          $sEnvFile, 
                                          $sCommand, 
                                          $sCommandArgs, 
                                          \$iExitCode, 
                                          $iTimeout, 
                                          \@lsCmdout ) ) {
        &logTrace ( 20342, 
        "<%s> Cmd [%s] with args [%s] executed successfully",
        $sMyName, $sCommand, $sCommandArgs );
      }
    }
  }
  #-----------------------------------------
  # Whack any persisting SAPComponents processes..
  #-----------------------------------------
  &logTrace ( 20054,
  "<%s> Killing SAPComponents processes for this instance", $sMyName );

  if ( SAPComponents::KillSAPComponentsProcesses ( $sInstProfile,
                                                   $sComponentType,
                                                   $sSAPAdmin ) ) {
    &logTrace ( 20055,
    "<%s> Processes for this instance do not exist after shutdown",
    $sMyName );
  }
  else {
    &logError ( 20056,
    "<%s> Could not remove all processes for this instance. Exiting",
    $sMyName );

    $iReturnCode = 0; # Failure..
  }
  
  #-----------------------------------
  # Remove preexisting IPC resources..
  #-----------------------------------
  #----------------------------------------
  # Define cleanipc command and arguments..
  #----------------------------------------
  my $iInstID = ();
  ($iInstID ) = $sInstName =~ /(\d{2})$/o;
  my $bSkipCleanipc = 0;
  $sCommand = "$sSAPHome/cleanipc";
  if (! -e $sCommand) {
    &logInfo ( 20191, "<%s> Binary file [%s] does not exist.", $sMyName, $sCommand);
    if ( $sComponentType =~ /^(ContentServerWD|Gateway)$/o ) {
      &logInfo ( 20192, "<%s> The agent will skip executing the command [%s] for Component [%s].", $sMyName, $sCommand, $sComponentType);
      $bSkipCleanipc = 1;
    }
    else {
      &logError ( 20193, "<%s> Binary file [%s] does not exist for Component [%s]. Returning failure.", $sMyName, $sCommand, $sComponentType);
      $iReturnCode = 0; # Failure.. 
    }
  #--------------------
  # Return, if failed..
  #--------------------
  $iReturnCode || return $iReturnCode;
  }
  if ( !$bSkipCleanipc) { 
    $sCommandArgs = "$iInstID remove";
    ( $iExitCode, $iTimeout, @lsCmdout ) = (); # Truncate..

    &logTrace ( 20057,
    "<%s> Stopping SAP Component with Command [%s] Args [%s]",
    $sMyName, $sCommand, $sCommandArgs );

    #------------------------
    # Request shutdown..
    #------------------------
    if ( ( $iTimeout = $iTillWhen - time ) <= 10 ) { # Get time bearings..
      &logError ( 20058,
      "<%s> Insufficient time to continue shutdown. Short by [%s] sec",
      $sMyName, abs ( $iTimeout ) );

      $iReturnCode = 0; # Failure..
    }
    elsif ( $iReturnCode =
            SAPComponents::ExecLocalCmd ( $sSAPAdmin,
                                          $sEnvFile,
                                          $sCommand,
                                          $sCommandArgs,
                                          \$iExitCode,
                                          $iTimeout,
                                          \@lsCmdout ) ) {
      &logTrace ( 20059,
      "<%s> Cmd [%s] with args [%s] executed successfully",
      $sMyName, $sCommand, $sCommandArgs );
    }
    else {
      &logError ( 20060,
      "<%s> Could not remove all IPC resources for User [%s]",
      $sMyName, $sSAPAdmin );
      $iReturnCode = 0; # Failure..
    }
  }
  my ($sKillFile, $sShutdownFile ) = ();

  #------------------
  # Remove kill.sap..
  #------------------
  $sKillFile = "/usr/sap/$sSAPSID/$sInstName/work/kill.sap";

  if ( -e $sKillFile ) {
    &logTrace ( 20061,
    "<%s> Located file [%s] for removal", $sMyName, $sKillFile );
    push ( @lsFiles, $sKillFile );
  }

  #----------------------
  # Remove shutdown.sap..
  #----------------------
  $sShutdownFile = "/usr/sap/$sSAPSID/$sInstName/work/shutdown.sap";

  if ( -e $sShutdownFile ) {
    &logTrace ( 20062,
    "<%s> Located file [%s] for removal", $sMyName, $sShutdownFile );
    push ( @lsFiles, $sShutdownFile );
  }

  #-------------------
  # Remove the files..
  #-------------------
  if ( scalar @lsFiles && ! Sys::UnlinkFilesAsUser ( @lsFiles ) ) {
    &logError ( 20063, "<%s> Failed to remove stale file(s) [%s]",
    $sMyName, join ( ' ', @lsFiles ) );
    $iReturnCode = 0; # Failure..
  }

  my $sResourceName = $ENV { VCS_LOG_RESOURCE_NAME };
  SAPComponents::DeletePidFile ( $sResourceName );

  return $iReturnCode;
}



=pod

 -------------------------------------------------------------------------
 Function : int GetResourceState ( IN String ComponentType,
                                   IN String SAPAdmin,
                                   IN String InstProfile );

 Description : First level monitor to determine state of resource.

 Parameters :

 $sComponentType            String identifier that classifies and describes the SAP Component.
 $sSAPAdmin                 The dedicated user for the SAP Components
 $sInstProfile              Specifies the full path and file name of the SAP Instance Profile.
 
 Returns : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 105  ONLINE with confidence level 50   VCSDef_MonitorExit::PARTIAL_ONLINE
 110  ONLINE with confidence level 100          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 -------------------------------------------------------------------------

=cut

sub GetResourceState {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sComponentType,
       $sSAPAdmin,
       $sInstProfile ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20064, "Subroutine <%s> called:
  ComponentType            [%s]
  SAPAdmin                 [%s]
  InstProfile              [%s]",

  $sMyName, $sComponentType, $sSAPAdmin, $sInstProfile );
  
  my @liDummyPids = ();
  my $iReturnCode = SAPComponents::GetProcState ( \@liDummyPids,
                                                  $sComponentType,
                                                  $sSAPAdmin,
                                                  $sInstProfile,
                                                  1 ); # Use FFLM..

  if ( VCSDef_MonitorExit::ONLINE == $iReturnCode ) {
    &logTrace ( 20065,
    "<%s> Found all required processes. Return code set to [%s]",
    $sMyName, $iReturnCode );
  }
  else {
    &logTrace ( 20066,
    "<%s> Did not find all required processes. Returning [%s]",
    $sMyName, $iReturnCode );
  }

  #--------------------------------
  # Populate the global PID array..
  #--------------------------------
  @liRunningProcs = @liDummyPids;
                                                                                
  &logTrace ( 20067,
  "<%s> Populating global running proclist with [%s] PIDs [%s]",
  $sMyName, scalar @liRunningProcs, join ( ' ', @liRunningProcs ) );

  return ( $iReturnCode );
}


=pod

 -------------------------------------------------------------------------
 Function : int GetProcState ( OUT ListReference OnlinePids,
                               IN String ComponentType,
                               IN String SAPAdmin,
                               IN String InstProfile,
                               IN Integer UseFFLM );

 Description : First level monitor to determine state of resource.

 Parameters :

 $rliOnlinePids             PIDs of relevant running processes
 $sComponentType            String identifier that classifies and describes the SAP Component.
 $sSAPAdmin                 The dedicated user for the SAP Components
 $sInstProfile              Specifies the full path and file name of the SAP Instance Profile.
 $iUseFFLM                  Filtering procedure
 
 Returns : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 105  ONLINE with confidence level 50   VCSDef_MonitorExit::PARTIAL_ONLINE
 110  ONLINE with confidence level 100          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; Statf ( $sKey eq 'sapstart' ) { # Check sapstart found ..
            $iSapstartPid = $iPid;
            unshift ( @$rliOnlinePids, $iPid ); # Insert at the front..
          }
e couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 -------------------------------------------------------------------------

=cut

sub GetProcState {
  #----------------
  # Get Arguments..
  #----------------
  my ( $rliOnlinePids,
       $sComponentType,
       $sSAPAdmin,
       $sInstProfile,
       $iUseFFLM ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20068, "Subroutine <%s> called with:
  OnlinePids               [%s]
  ComponentType            [%s]
  SAPAdmin                 [%s]
  InstProfile              [%s]
  UseFFLM                  [%s]",

  $sMyName, $rliOnlinePids, $sComponentType, $sSAPAdmin, 
  $sInstProfile, $iUseFFLM );

  #-----------------------------
  # Set return code to success..
  #-----------------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE; # Success..

  #----------------------
  # Initialise Variables..
  #----------------------
  my ( $iProcessCount, $iUseRFLM, $iFound, $iUid, $iPid, $sLang,
       $sKey, $rhlProc, $sPidFile, $sProc, $sTemp,
       @hProcListHash, @lsUpdates, @lsValues,
       %hiProcPids, %hsSrvrDetails, %hsCopy ) = ();

  @$rliOnlinePids = ();
  

  #------------------------------------
  # Attempt to get monitoring details..
  #------------------------------------
  if ( ( 0 == ( $iUid = getpwnam $sSAPAdmin ) ) && # User is the super-user..
       ( 2 == $iUseFFLM ) ) {                  # And called to kill-by-UID..
    &logTrace ( 20069, "<%s> Unsafe to kill processes by User [%s] UID [%s].
    Using filter-based approach.", $sMyName, $sSAPAdmin, $iUid ),
    $iUseFFLM = 1;
  }

  if ( $iUseFFLM != 2 ) { # Called for first level check..
    if ( SAPComponents::GetSrvrDetails ( $sComponentType, 
                                         $sInstProfile,
                                         \%hsSrvrDetails ) ) {
      &logTrace ( 20070,
      "<%s> Got details for monitoring SAPComponents instance", $sMyName );
  
      %hsCopy = %hsSrvrDetails;        # Backup..
      $iUseRFLM = scalar keys %hsCopy; # Set to entire count..
    }
    else {
      &logError ( 20071,
      "<%s> Failed to get details for monitoring SAPComponents instance",
      $sMyName );
      return VCSDef_MonitorExit::UNKNOWN;
    }
                                  
    #---------------------------------
    # Check if we attempt to be fast.. 
    #---------------------------------
    if ( $iUseFFLM ) { # FFLM..
      $iUseRFLM = 0;   # Hopefully skip..

      foreach $sKey ( keys %hsCopy ) {          # Use copy..
        @lsUpdates = ();                        # Reset..
        @lsValues = @{ $hsCopy { $sKey } };     # Get values..
        $sTemp .= "\nSAPComponents process [$sKey]\n"; # Display string..

        #----------------
        # Read in pairs..
        #----------------
        while ( ( $sPidFile, $sProc ) = splice ( @lsValues, 0, 2 ) ) {

          #-----------
          # Run FFLM..
          #-----------
          $iFound = SAPComponents::FFLMStatus ( $sPidFile,
                                                $sProc,
                                                $iUid,
                                                \$iPid );

          #----------------
          # Update status..
          #----------------
          push ( @lsUpdates, $sPidFile, $sProc, $iFound );

          $sTemp .= "PID File     [$sPidFile]\n";
          $sTemp .= "Srch Process [$sProc]\n";
          $sTemp .= "FFLM Status  [$iFound]\n\n";
        }

        @{ $hsCopy { $sKey } } = @lsUpdates;

        #------------------------
        # Handle monitor status..
        #------------------------
        if ( VCSDef_MonitorExit::ONLINE == $iFound ) {
          push ( @$rliOnlinePids, $iPid ); # Push..
        }
        else {
          ++ $iUseRFLM; # Mark it for RFLM..
        }
      }

      #----------------
      # Print summary..
      #----------------
      &logTrace ( 20072,
      "<%s> SAPComponents instance process details ---%s\n\n",
      $sMyName, $sTemp );

      ( $sTemp, $sPidFile, $sProc ) = (); # Release memory..
    }
    else { # Use RLFM..
      #-----------------------------------------
      # Mark status of each service as offline..
      #-----------------------------------------
      $iFound = VCSDef_MonitorExit::OFFLINE; # Set to offline..
      foreach $sKey ( keys %hsCopy ) {
        @lsUpdates = (); # Reset..

        @lsValues = @{ $hsCopy { $sKey } }; # Get values..
        while ( scalar @lsValues ) {
          push ( @lsUpdates, splice ( @lsValues, 0, 2 ), $iFound );
        }

        @{ $hsCopy { $sKey } } = @lsUpdates; # Update..
      }
    }
  }
  else { # Called for Kill..
    &logTrace ( 20073,
    "<%s> Called to get processes for User [%s] UID [%s]",
    $sMyName, $sSAPAdmin, $iUid );
    ++ $iUseRFLM;
  }

  #-----------------------
  # Regular slow :-( FLM..
  #-----------------------
  if ( $iUseRFLM ) {
    &logTrace ( 20074,
    "<%s> Running regular first level monitor", $sMyName );

    #----------------------------------
    # Set LANG for better performance..
    #----------------------------------
    $sLang = $ENV { LANG }; # Backup..
    $ENV { LANG } = 'C';    # Now set..

    #-----------------------------------
    # Call processes and store in hash..
    #-----------------------------------
    $iProcessCount = Proc::GetLongProcessListHash ( \@hProcListHash );

    #----------
    # Restore..
    #----------
    defined $sLang ? $ENV { LANG } = $sLang : delete $ENV { LANG }; # Restore..

    #----------------------
    # Check if we got any..
    #----------------------
    if ( $iProcessCount ) {
      &logTrace ( 20075,
      "<%s> Got [%s] processes from proc table for filtering",
      $sMyName, $iProcessCount );
    }
    else {
      &logError ( 20076,
      "<%s> Call to get process list did not return any processes",
      $sMyName );
      return VCSDef_MonitorExit::OFFLINE; # Offline..
    }

    #-----------------------
    # Search for processes..
    #-----------------------
    foreach $rhlProc ( @hProcListHash ) {
      #--------------------------------------------------------------
      # Each $rhlProc is a ref. to a hash with the following keys..
      # "UID"     =>  <user id>
      # "PID"     =>  <process id>
      # "PPID"    =>  <parent process id>
      # "COMMAND" =>  <command string>
      #--------------------------------------------------------------

      next unless defined $$rhlProc { UID }; # Bug in Proc::GetLongProcessListHash..
      next if $$rhlProc { PID } == $$;       # Skip self..
      $iPid = $$rhlProc { PID };             # Assign..

      &logTrace ( 20077, "<%s> Searching UID [%s] PID [%s] PPID [%s] CMD [%s]",
      $sMyName, $$rhlProc { UID }, $$rhlProc { PID }, $$rhlProc { PPID },
      $$rhlProc { COMMAND } );

      if ( $$rhlProc { UID } == $iUid ) { # Process owner..
        #-----------------------------------------------
        # Look for processes that were offline in FFLM..
        #-----------------------------------------------
        if ( $iUseFFLM != 2 ) { # Request to filter..
          foreach $sKey ( keys %hsCopy ) {
            @lsUpdates = (); # Reset..
            @lsValues = @{ $hsCopy { $sKey } }; # Get values..
    
            #--------------------------------------------------
            # Should have details per three elements of array..
            # <PIDFILE> <PROCESS> <STATE>
            #--------------------------------------------------
            while ( ( $sPidFile, $sProc, $iFound ) =
                      splice ( @lsValues, 0, 3 ) ) {
              if ( VCSDef_MonitorExit::OFFLINE == $iFound ) { # Earlier marked offline..
                if ( $$rhlProc { COMMAND } =~ /$sProc/ ) {    # Now found..
                  unshift ( @$rliOnlinePids, $iPid );         # Store..
                  
                  if( $$rhlProc { COMMAND } =~ /sapstartsrv/o ){
                    $iSapStartSrv = $iPid; # Populate global..
                  }

                  &logTrace ( 20078,
                  "<%s> Found PID [%s] PPID [%s] CMD [%s] with filter [%s]",
                  $sMyName, $iPid, $$rhlProc { PPID }, $$rhlProc { COMMAND },
                  $sProc );

                  $hiProcPids { $sKey } = $iPid;        # Store..
                  $iFound = VCSDef_MonitorExit::ONLINE; # Mark online..
                }
              }
    
              #----------------
              # Update status..
              #----------------
              push ( @lsUpdates, $sPidFile, $sProc, $iFound );
            }
    
            @{ $hsCopy { $sKey } } = @lsUpdates; # Update..
          }
        }
        else { # Request for kill..
          &logTrace ( 20079,
          "<%s> Pushing PID [%s] with PPID [%s] Cmd [%s] on stack",
          $sMyName, $iPid, $$rhlProc { PPID }, $$rhlProc { COMMAND } );
          push ( @$rliOnlinePids, $iPid ); # Store..
        } 
      }
      
    }
  }

  #------------------------------------
  # Set final status for each process..
  #------------------------------------
  my ( $iOnline, $iOffline, $sEntryPointName ) = ();

  ( $sEntryPointName = $0 ) =~ s#.*/##; # Basename..
  $sTemp = ''; # Ensure..

  foreach $sKey ( keys %hsCopy ) {
    @lsValues = @{ $hsCopy { $sKey } }; # Get values..
    $sTemp .= "\nProcess      [$sKey]\n";

    while ( ( $sPidFile, $sProc, $iFound ) =
              splice ( @lsValues, 0, 3 ) ) {
      #-----------------------------------
      # Create PID files for use in FFLM..
      #-----------------------------------
      if ( $iUseRFLM                                 &&      # In RFLM..
           $iUseFFLM != 2                            &&      # Asked to filter..
           $hiProcPids { $sKey }                     &&      # PID populated..
           ( $sEntryPointName =~ /^(monitor|online)$/o ) ) { # In monitor or online..

        &logTrace ( 20080,
        "<%s> Creating PID file [%s] with PID [%s]",
        $sMyName, $sPidFile, $hiProcPids { $sKey } );

        #---------
        # Create..
        #---------
        if ( SAPComponents::CreatePidFile ( $sPidFile,
                                            $hiProcPids { $sKey } ) ) {
          &logTrace ( 20081,
          "<%s> Successfully created PID file [%s]", $sMyName, $sPidFile );
        }
        else {
          &logError ( 20082,
          "<%s> Could not create PID file [%s] successfully",
          $sMyName, $sPidFile );
        }
      }

      #-------------
      # Set status..
      #-------------
      if ( VCSDef_MonitorExit::ONLINE == $iFound ) {
        $iOnline = $iFound;
      }
      elsif ( VCSDef_MonitorExit::OFFLINE == $iFound ) {
        $iOffline = $iFound;
      }

      $sTemp .= "Srch Process [$sProc]\n";
      $sTemp .= "FLM Status   [$iFound]\n\n";
    }

    $sTemp .= "--------------------";
  }

  #-------------------
  # Set final status..
  #-------------------
  $iReturnCode = $iOffline || $iOnline || VCSDef_MonitorExit::UNKNOWN;

  #------------------------------------------------
  # Reset status based on PIDs, if not in monitor..
  #------------------------------------------------
  if ( $sEntryPointName ne 'monitor'                   &&
       ( VCSDef_MonitorExit::OFFLINE == $iReturnCode ) &&
       scalar @$rliOnlinePids ) {
    $iReturnCode = VCSDef_MonitorExit::PARTIAL_ONLINE;

    &logTrace ( 20083,
    "<%s> Found [%s] PIDs [%s] for this instance of SAPComponents.
    Resetting return code to [%s]",
    $sMyName, scalar @$rliOnlinePids, join ( ' ', @$rliOnlinePids ),
    $iReturnCode );
  }

  #-----------------------
  # Report on FLM status..
  #-----------------------
  if ( $iUseFFLM != 2 ) { # Request to filter..
    &logTrace ( 20084,
    "<%s> SAPComponents process details ---\n%s\nReturn code is [%s]",
    $sMyName, $sTemp, $iReturnCode );
    $sTemp = undef; # Release..
  }

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : int Monitor ( IN String ComponentType,
                          IN String EnvFile,
                          IN String InstProfile,
                          IN String MonitorProgram,
                          IN String SAPAdmin,
                          IN String SAPHome,
                          IN String InstName,
                          IN String SAPHost
                          IN Integer SecondLevelMonitor );

 Description: Performs the in-depth monitoring

 Parameters :

 $sComponentType            String identifier that classifies and describes the SAP Component.
 $sEnvFile                  Absolute path to environment file to source
 $sInstProfile              Specifies the full path and file name of the SAP Instance Profile.
 $sMonitorProgram           Full path to external monitor program
 $sSAPAdmin                 The dedicated user for the SAP Components
 $sSAPHome                  SAP Instance Home
 $sInstName                 SAP Instance Name
 $sSAPHost                  SAP Instance virtual host name
 $iSecondLevelMonitor       Cycle count to run in-depth monitoring
 
 Returns : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 110  ONLINE with confidence level 100          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 ??   HUNG: External monitor timed out         VCSDef_MonitorExit::TIMEOUT

 The entry point hangs if in-depth monitoring cannot be completed.
 -------------------------------------------------------------------------

=cut

sub Monitor {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sComponentType,
       $sEnvFile,
       $sInstProfile,
       $sMonitorProgram,
       $sSAPAdmin,
       $sSAPHome,
       $sInstName,
       $sSAPHost,
       $iSecondLevelMonitor ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20085, "Subroutine <%s> called with:
  ComponentType            [%s]
  EnvFile                  [%s]
  InstProfile              [%s]
  MonitorProgram           [%s]
  SAPAdmin                 [%s]
  SAPHome                  [%s]
  InstName                 [%s]
  SAPHost                  [%s]
  SecondLevelMonitor       [%s]",

  $sMyName, $sComponentType,$sEnvFile, $sInstProfile, $sMonitorProgram, 
  $sSAPAdmin, $sSAPHome, $sInstName, $sSAPHost, $iSecondLevelMonitor );

  #-----------------------------
  # Set return code to success..
  #-----------------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE; # Success..

  my $iTillWhen = undef;

  #----------------------------------------------
  # In-depth monitoring, if correct time to run..
  #----------------------------------------------
  if ( VCSagentFW::RunSecondLevelMonitor ( $iSecondLevelMonitor ) ) {
    #--------------------------
    # Just In Time Validation..
    #--------------------------
    if ( VCSDef_MonitorExit::ONLINE == ( $iReturnCode =
         SAPComponents::DelayedAttributeCheck ( $sEnvFile,
                                                $sInstProfile,
                                                $sSAPAdmin ) ) ) {
      &logTrace ( 20086,
      "<%s> Delayed attribute check completed successfully",
      $sMyName );
    }
    else {
      &logError ( 20087,
      "<%s> Attribute validation failed. Returning [%s]",
      $sMyName, $iReturnCode );
      return $iReturnCode;
    }

    &logTrace ( 20088,
    "<%s> Proceeding with second level monitoring", $sMyName );

    #-------------------------------------------------------
    # Upper bound..
    # Time in EPOCH when entry pt started + VCS time limit..
    #-------------------------------------------------------
    $iTillWhen = $^T + SAPComponents::GetEngineTimeout ( 1 );

    if ( VCSDef_MonitorExit::ONLINE == ( $iReturnCode = 
         SAPComponents::SecondLevelMonitor ( $sComponentType, # Running SLM..
                                             $sEnvFile,
                                             $sInstProfile,
                                             $sSAPAdmin,
                                             $sSAPHome,
                                             $sInstName,
                                             $sSAPHost,
                                             $iTillWhen ) ) ) {

      &logTrace ( 20089,
      "<%s> SAPComponents instance online with status [%s] after SLM",
      $sMyName, $iReturnCode );
    }
    else {
      &logError ( 20090,
      "<%s> SAPComponents instance NOT online after SLM. Status [%s]",
      $sMyName, $iReturnCode );
      return $iReturnCode;
    }
  }

  #------------------------------------------------
  # Exit if external monitor has not been defined..
  #------------------------------------------------
  return $iReturnCode unless defined $sMonitorProgram;

  #---------------------------------------
  # MonitorProgram defined..
  # Separate the executable and the args..
  #---------------------------------------
  my ( $sExtCmd, $sExtCmdArgs ) = $sMonitorProgram # Separate
                                =~ /(\S+)\s*(.*)/; # Cmd and args..

  $sExtCmdArgs = '' unless defined $sExtCmdArgs;   # Prevent undefs..

  if ( $sExtCmd ) {                                # Defined..
    #----------------------------------------
    # Get time bearings, if not got earlier..
    #----------------------------------------
    $iTillWhen ||= $^T + SAPComponents::GetEngineTimeout ( 1 );

    if ( VCSDef_MonitorExit::ONLINE ==             # and found online..
         ( $iReturnCode =                          # by the status..
         SAPComponents::ExternalMonitor ( $sSAPAdmin,
                                          $sEnvFile,
                                          $sExtCmd,
                                          $sExtCmdArgs,
                                          $iTillWhen ) ) ) {
      &logTrace ( 20091,
      "<%s> SAPComponents instance found online with status [%s]
      after executing external monitor [%s] with args [%s]",
      $sMyName, $iReturnCode, $sExtCmd, $sExtCmdArgs );
    }
    else {
      &logError ( 20092,
      "<%s> SAPComponents instance NOT online after executing external monitor
      [%s] with args [%s]. Status [%s]",
      $sMyName, $sExtCmd, $sExtCmdArgs, $iReturnCode );
    }
  }

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : int DelayedAttributeCheck ( IN String EnvFile,
                                        IN String InstProfile,
                                        IN String SAPAdmin );

 Description : Just In Time Validation

 Parameters :

 $sEnvFile                  Absolute path to environment file to source
 $sInstProfile              Specifies the full path and file name of the SAP Instance Profile.
 $sSAPAdmin                 The dedicated user for the SAP Components
 
 Returns  : Integer value as defined below:

 110  ONLINE with confidence level 110          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 -------------------------------------------------------------------------

=cut

sub DelayedAttributeCheck {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sEnvFile,
       $sInstProfile,
       $sSAPAdmin ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20093, "Subroutine <%s> called with:
  EnvFile                  [%s]
  InstProfile              [%s]
  SAPAdmin                 [%s]",

  $sMyName, $sEnvFile, $sInstProfile, $sSAPAdmin );

  #-----------------------------
  # Set return code to success..
  #-----------------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE; # Success..

  #--------------------------------------------
  # Create hash to weed out empty args values..
  #--------------------------------------------
  my @lsArgs = qw ( EnvFile InstProfile SAPAdmin );
  my @lsReqdArgs = qw ( EnvFile InstProfile SAPAdmin );
  my ( @lsMissing, %hsArgList ) = ();

  @hsArgList { @lsArgs } = @_; # Assign..

  #-----------------------
  # Check what's missing..
  #-----------------------
  if ( @lsMissing = grep { $hsArgList { $_ } !~ /\S/o } @lsReqdArgs ) {
    &logError ( 20094,
    "<%s> The following mandatory fields were not set [%s], Total [%s]",
    $sMyName, join ( ' ', @lsMissing ), scalar @lsMissing );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN; # Failure..
  }

  #--------------------
  # Return, if failed..
  #--------------------
  if ( VCSDef_MonitorExit::ONLINE != $iReturnCode ) {
    return $iReturnCode;
  }

  #----------------------------------
  # Validate parameter InstProfile..
  #----------------------------------
  if ( ! -e $sInstProfile ) {
    &logError ( 20095,
    "<%s> InstProfile [%s] file does not exist",
    $sMyName, $sInstProfile );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN; # Failure..
  }

  #------------------------------
  # Validate parameter EnvFile..
  #------------------------------
  if ( ! -e $sEnvFile ) {
    &logError ( 20096,
    "<%s> EnvFile [%s] file does not exist",
    $sMyName, $sEnvFile );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN; # Failure..
  }

  #-------------------------------
  # Validate parameter SAPAdmin..
  #-------------------------------
  if ( ! defined getpwnam $sSAPAdmin ) {
    &logError ( 20097,
    "<%s> SAPAdmin [%s] is not a valid UNIX login",
    $sMyName, $sSAPAdmin );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN; # Failure..
  }

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : int SecondLevelMonitor ( IN String ComponentType,
                                     IN String EnvFile,
                                     IN String InstProfile,
                                     IN String SAPAdmin,
                                     IN String SAPHome,
                                     IN String InstName,
                                     IN String SAPHost,
                                     IN Integer TillWhen );

 Description : Does in-depth monitoring, if enabled.

 Parameters :
 
 $sComponentType            String identifier that classifies and describes the SAP Component.
 $sEnvFile                  Absolute path to environment file to source
 $sInstProfile              Specifies the full path and file name of the SAP Instance Profile.
 $sSAPAdmin                 The dedicated user for the SAP Components
 $sSAPHome                  SAP Instance Home
 $sInstName                 SAP Instance Name
 $sSAPHost                  SAP Instance virtual host name
 $iTillWhen                 Time in EPOCH left for IInd level monitoring
 
 Returns  : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 110  ONLINE with confidence level 110          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 ??   HUNG: External monitor timed out         VCSDef_MonitorExit::TIMEOUT

 The entry point hangs if the command cannot be completed.
 -------------------------------------------------------------------------

=cut

sub SecondLevelMonitor {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sComponentType,
       $sEnvFile,
       $sInstProfile,
       $sSAPAdmin,
       $sSAPHome,
       $sInstName,
       $sSAPHost,
       $iTillWhen ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20098, "Subroutine <%s> called with:
  ComponentType            [%s]
  EnvFile                  [%s]
  InstProfile              [%s]
  SAPAdmin                 [%s]
  SAPHome                  [%s]
  InstName                 [%s]
  SAPHost                  [%s]
  TillWhen                 [%s]",

  $sMyName, $sComponentType, $sEnvFile, $sInstProfile, $sSAPAdmin, 
  $sSAPHome, $sInstName, $sSAPHost,$iTillWhen );  

  #------------------
  # Hopeful success..
  #------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE;

  #-------------------------
  # Define local variables..
  #-------------------------
  my ( $iTimeout, $iExitCode, $sCommand, $sCommandArgs, $iInstID, @lsCmdout ) = ();
  
  ($iInstID ) = $sInstName =~ /(\d{2})$/o;
  #---------------------------------------------
  # Declare monitor command and args variables..
  #---------------------------------------------
  $sCommand = "$sSAPHome/sapcontrol";
  $sCommandArgs = "-host $sSAPHost -nr $iInstID -function GetProcessList";

  #-------------------------------
  # Validate and execute command..
  #-------------------------------
  if ( ( $iTimeout = $iTillWhen - time ) <= 10 ) { # Get time bearings..
    &logError ( 20099,
    "<%s> Insufficient time to continue in-depth monitoring.
    Short by [%s] sec",
    $sMyName, abs ( $iTimeout ) );

    $iReturnCode = VCSDef_MonitorExit::UNKNOWN; # Failure..
  }
  elsif ( $sComponentType =~ /^(ContentServerWD|Gateway|WebDispatcher|MDM)$/o ) {
    $iReturnCode = SAPComponents::ExecSAPControl ( $sComponentType,
                                                   $sSAPAdmin,
                                                   $sEnvFile,
                                                   $sCommand,
                                                   $sCommandArgs,
                                                   $iTimeout );
  }

  return $iReturnCode;
}

=pod

 -------------------------------------------------------------------------
 Function : int ExecSAPControl ( IN String ComponentType,
                                 IN String SAPAdmin,
                                 IN String EnvFile,
                                 IN String Cmd,
                                 IN String CmdArgs,
                                 IN Integer Timeout );

 Description : Decides on status of SAP Component based on sapcontrol command.

 Parameters :
 
 $sComponentType      String identifier that classifies and describes the SAP Component.
 $sSAPAdmin           UNIX user dedicated for this instance of SAP
 $sEnvFile            Full path to the environment file for SAP
 $sCmd                Full path to jsmon binary
 $sCmdArgs            Argument list for jsmon
 $iTimeout            Time in seconds for execution

 Returns  : Integer value as defined below: 

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 110  ONLINE with confidence level 110          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 -------------------------------------------------------------------------

=cut


sub ExecSAPControl {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sComponentType,
       $sSAPAdmin,
       $sEnvFile,
       $sCmd,
       $sCmdArgs,
       $iTimeout ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20100, "Subroutine <%s> called with:
  ComponentType [%s]
  SAPAdmin      [%s]
  EnvFile       [%s]
  Cmd           [%s]
  CmdArgs       [%s]
  Timeout       [%s]",

  $sMyName, $sComponentType,
  $sSAPAdmin, $sEnvFile, $sCmd, $sCmdArgs, $iTimeout );

  #-------------------------------------------
  # Set return code optimistically to online..
  #-------------------------------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE;

  #-------------------------
  # Define local variables..
  #-------------------------
  my ( $iExitCode, @lsCmdOut ) = ();

  #-------------------
  # Validate command..
  #-------------------
  if ( ! -e $sCmd ) {
    &logError ( 20101,
    "<%s> SAP Cmd [%s] does not exist", $sMyName, $sCmd );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
  }
  elsif ( ! -x $sCmd ) {
    &logError ( 20102,
    "<%s> SAP Cmd [%s] is not executable", $sMyName, $sCmd );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
  }
  elsif ( $iTimeout < 10 ) {
    &logError ( 20103,
    "<%s> Insufficient time [%s] to execute Cmd [%s] with args [%s]",
    $sMyName, $iTimeout, $sCmd, $sCmdArgs );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
  }
  $iReturnCode = SAPComponents::ExecLocalCmd ( $sSAPAdmin, # User context..
                                               $sEnvFile,  # Use envfile..
                                               $sCmd,
                                               $sCmdArgs,
                                               \$iExitCode,
                                               $iTimeout,
                                               \@lsCmdOut );

  if( defined $iExitCode && 3 == $iExitCode ){
    #-------------------------------
    # Command execution successful..
    #-------------------------------
    &logTrace ( 20104,
    "<%s> Cmd [%s] with args [%s] ran successfully with exit code [%s]",
    $sMyName, $sCmd, $sCmdArgs, $iExitCode );

    #------------------------------------------------------
    # Get status based on state of dispatcher and servers..
    #------------------------------------------------------
      $iReturnCode = SAPComponents::GetCompStatus ( $sComponentType, \@lsCmdOut );
      @lsCmdOut = (); # Free..
  }
  elsif ( ! defined $iExitCode ) { # Time-out..
    $lsCmdOut [0] ||= 'undef'; # Set if not defined..
    $iReturnCode = $lsCmdOut [0] eq 'timeout'? VCSDef_MonitorExit::TIMEOUT: # Timeout..
                                               VCSDef_MonitorExit::UNKNOWN; # Failure..

    &logError ( 20105,
    "<%s> Cmd [%s] with args [%s] timed out in [%s] sec. Returning [%s]",
    $sMyName, $sCmd, $sCmdArgs, $iTimeout, $iReturnCode );
  }
  elsif( 4 == $iExitCode || 1 == $iExitCode ) {
    $iReturnCode = VCSDef_MonitorExit::OFFLINE;
    &logError ( 20106,
    "<%s> Cmd[%s] executed successfully with exit code [%s] indicating  all processes stopped",
    $sMyName, $sCmd, $iExitCode );
  }
  else {
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
    &logError ( 20107,
    "<%s> Cmd [%s] failed with exit code [%s]. Cmd output [%s]
    Returning [%s]", $sMyName, $sCmd, $iExitCode,
    join ( chr 012, @lsCmdOut ), $iReturnCode );
  }

  return $iReturnCode;
}

=pod

 ---------------------------------------------------------------------------------------
 Function : int GetCompStatus ( IN String ComponentType,
                                 IN RefStrList CmdOut );

 Description : Decides on status of SAP Instance based on given sapcontrol
               output.

 Parameters :

 $sComponentType      String identifier that classifies and describes the SAP Component.
 $rlsCmdOut      Reference to list containing output from jsmon execution. 

 Returns  : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 110  ONLINE with confidence level 110          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 ---------------------------------------------------------------------------------------

=cut

sub GetCompStatus {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sComponentType, 
       $rlsCmdOut ) = @_;
  
  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20108, "Subroutine <%s> called with:
  ComponentType     [%s]
  rlsCmdOut         [%s]",

  $sMyName, $sComponentType, $rlsCmdOut );

  #-------------------------------------------
  # Set return code optimistically to online..
  #-------------------------------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE;

  #------------------------------
  # Validate against empty list..
  #------------------------------
  return VCSDef_MonitorExit::UNKNOWN unless scalar @$rlsCmdOut;

  #-------------------------
  # Define local variables..
  #-------------------------
  my ( $sTmp, $sKey ) = ( '' ) x 2;
  my ( $iSrv, $iRunning ) = ( 0 ) x 2;
  my %hsCmdOut = ();
  foreach (  @$rlsCmdOut ) {
    if( $sComponentType =~ /^Gateway$/o ) { 
      if ( $_ =~ /\b(Gateway)\b\,\s+(.*?)\s*\d/io &&
           $2 =~ /Green\,\s+running/io ) {
        ++ $iSrv;
        ++ $iRunning;
        &logTrace ( 20109, "<%s> Gateway is in running state",$sMyName );
        last;
      }
    }
    elsif( $sComponentType =~ /^ContentServerWD$/o ) { 
      if ( $_ =~ /Green\,\s+running/io ) {
        ++ $iSrv;
        ++ $iRunning;
        &logTrace ( 20109, "<%s> ContentServer Web Dispatcher is in running state",$sMyName );
        last;
      }
    }
    if( $sComponentType =~ /^MDM$/o ) { 
      if ( $_ =~ /\b(mds|mdis|mdss)\b\,\s+(.*?)\s*\d/io &&
           $2 =~ /Green\,\s+running/io ) {
        ++ $iSrv;
        ++ $iRunning;
        &logTrace ( 20343, "<%s> MDM is in running state",$sMyName );
        last;
      }
    }
    else {
      if ( $_ =~ /\b(Web\s+Dispatcher)\b\,\s+(.*?)\s*\d/io &&
           $2 =~ /Green\,\s+running/io ) {
        ++ $iSrv;
        ++ $iRunning;
        &logTrace ( 20110, "<%s> Web Dispatcher is in running state",$sMyName );
        last;
      }
    }
  }

  #-----------------
  # Check on state..
  #-----------------
  if ( $iRunning ) {                            # One server running..
    &logTrace ( 20111,
    "<%s> Found [%s] of [%s] server(s) running. Returning [%s]",
    $sMyName, $iRunning, $iSrv, $iReturnCode );
  }
  else {
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
    &logWarn ( 20112,
    "<%s> Status of SAP instance undecided:\n%sReturning [%s]",
    $sMyName, $sTmp, $iReturnCode );
  }

  ( $sTmp, $sKey ) = (); # Free..

  return $iReturnCode;
}

=pod

 -------------------------------------------------------------------------
 Function : int ExternalMonitor ( IN String SAPAdmin,
                                  IN String EnvFile,
                                  IN String Cmd,
                                  IN String CmdArgs,
                                  IN Integer TillWhen );

 Description : Handles user defined external monitor program

 Parameters :

 $sSAPAdmin                 The dedicated user for the SAP Components
 $sEnvFile                  Absolute path to environment file to source
 $sCmd                      User defined utilty to execute
 $sCmdArgs                  Command line arguments
 $iTillWhen                 Time in EPOCH left for IInd level monitoring
 
 Returns  : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 110  ONLINE with confidence level 110          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 ??   HUNG: External monitor timed out         VCSDef_MonitorExit::TIMEOUT

 The entry point hangs if the command cannot be completed.
 -------------------------------------------------------------------------

=cut

sub ExternalMonitor {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sSAPAdmin,
       $sEnvFile,
       $sCmd,
       $sCmdArgs,
       $iTillWhen ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20113, "Subroutine <%s> called with:
  SAPAdmin                 [%s]
  EnvFile                  [%s]
  Cmd                      [%s]
  CmdArgs                  [%s]
  TillWhen                 [%s]",

  $sMyName, $sSAPAdmin, $sEnvFile, $sCmd, $sCmdArgs, $iTillWhen );

  #------------------
  # Hopeful success..
  #------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE;

  #------------------------
  # Setup local variables..
  #------------------------
  my ( $iExitCode, $iTimeout, @lsCmdout ) = ();
  my $iBuf = 5; # Secs..

  #-------------------------
  # Handled exit code hash..
  #-------------------------
  my %hiExitCodes = (  0, VCSDef_MonitorExit::ONLINE,
                     110, VCSDef_MonitorExit::ONLINE,
                       1, VCSDef_MonitorExit::OFFLINE,
                     100, VCSDef_MonitorExit::OFFLINE,
                          VCSDef_MonitorExit::TIMEOUT,
                          VCSDef_MonitorExit::TIMEOUT );

  #---------------------------------------------
  # Check if command can be found and executed..
  #---------------------------------------------
  if ( ! -e $sCmd ) {
   &logError ( 20114,
    "<%s> External monitor [%s] does not exist",
    $sMyName, $sCmd );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
  }
  elsif ( ! -x $sCmd ) {
   &logError ( 20115,
    "<%s> External monitor [%s] not executable.",
    $sMyName, $sCmd );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
  }
  elsif ( $sEnvFile =~ /\S/ && ! -e $sEnvFile ) {
    &logError ( 20116,
    "<%s> EnvFile [%s] does not exist", $sMyName, $sEnvFile );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
  }
elsif ( ( $iTimeout = $iTillWhen - ( time + $iBuf ) ) <= 5 ) { # Lack of time..
   &logError ( 20117,
    "<%s> Insufficient time to proceed with external monitor [%s].
    Short by [%s] sec", $sMyName, $sCmd, abs ( $iTimeout ) );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
  }

  #--------------------------------------------
  # Return if validation failed at this point..
  #--------------------------------------------
  unless ( VCSDef_MonitorExit::ONLINE == $iReturnCode ) {
    return $iReturnCode;
  }

  &logTrace ( 20118,
  "<%s> Executing external monitor with Command [%s] Args [%s] in [%s] secs",
  $sMyName, $sCmd, $sCmdArgs, $iTimeout );

  #----------------------
  # Execute the command..
  #----------------------
  SAPComponents::ExecLocalCmd ( $sSAPAdmin, 
                                $sEnvFile, 
                                $sCmd, 
                                $sCmdArgs, 
                                \$iExitCode, 
                                $iTimeout, 
                                \@lsCmdout );
                               
  #------------------------
  # Report on exit status..
  #------------------------
  if ( ! defined $iExitCode ) {
    $lsCmdout [0] ||= 'undef'; # Set if not defined..
    $iExitCode = $lsCmdout [0] eq 'timeout'? VCSDef_MonitorExit::TIMEOUT: # Timeout..
                                             VCSDef_MonitorExit::UNKNOWN; # Failure..
  }
  elsif ( $hiExitCodes { $iExitCode } &&
          VCSDef_MonitorExit::ONLINE ==
          $hiExitCodes { $iExitCode } ) {
    &logTrace ( 20119,
    "<%s> Command [%s] with args [%s] executed successfully with exit code [%s].
    Cmd output [%s]",
    $sMyName, $sCmd, $sCmdArgs, $iExitCode, join ( chr 012, @lsCmdout ) );
  }
  else {
    &logWarn ( 20120,
    "<%s> Command [%s] with args [%s] execution failed with exit code [%s].
    Cmd output [%s]",
    $sMyName, $sCmd, $sCmdArgs, $iExitCode, join ( chr 012, @lsCmdout ) );
  }

  #--------------------------------
  # Setup appropriate return code..
  #--------------------------------
  $iReturnCode = $hiExitCodes { $iExitCode } ||
                 VCSDef_MonitorExit::UNKNOWN;

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : bool CreatePidFile ( IN String PidFile,
                                 In Integer Pid );


 Description : Creates PID file with given PID

 Parameters :

 $sPidFile     File to create
 $iPid         PID to store

 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub CreatePidFile {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sPidFile,
       $iPid ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20121, "Subroutine <%s> called with:
  PidFile         [%s]
  Pid             [%s]",

  $sMyName, $sPidFile, $iPid );

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1;

  #-----------------------
  # Check against undefs..
  #-----------------------
  unless ( defined $iPid && $iPid =~ /^\d+$/o) {
    $iPid ||= 'undef'; # Prevent undefs.
    &logError ( 20122, "<%s> PID [%s] is not a valid positive number",
    $sMyName, $iPid );
    return 0; # Proceed no further..
  }

  #------------------
  # Create PID file..
  #------------------
  &logTrace ( 20123,
  "<%s> Creating PID file [%s] with PID [%s]",
  $sMyName, $sPidFile, $iPid );

  if ( sysopen ( PIDFILE, $sPidFile, O_WRONLY|O_SYNC|O_CREAT|O_TRUNC,
                 0600 ) ) {
    print PIDFILE $iPid, "\n"; # Update..

    unless ( close ( PIDFILE ) ) {
      &logWarn ( 20124,
      "<%s> Failed to close PidFile [%s] after update: [%s]",
      $sMyName, $sPidFile, $! );
      $iReturnCode = 0;
    }
  }
  else {
    &logWarn ( 20125,
    "<%s> Failed to create PID file [%s] with PID [%s]",
    $sMyName, $sPidFile, $iPid );
    $iReturnCode = 0;
  }

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : int FFLMStatus ( IN String PidFile,
                             IN String Proc,
                             IN Integer Uid,
                             OUT RefInt Pid );

 Description: Checks status of Process listed in PidFile, if running

 Parameters :

 $sPidFile       Absolute path to PID file of service
 $sProc          Process string to search for in proc table
 $iUid           UNIX UID of owner
 $riPid          Reference to existing PID

 Returns : Integer value as defined below:
 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 110  ONLINE with confidence level 100          VCSDef_MonitorExit::ONLINE
 -------------------------------------------------------------------------

=cut

sub FFLMStatus {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sPidFile,
       $sProc,
       $iUid,
       $riPid ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20126, "Subroutine <%s> called with:
  PidFile                [%s]
  Proc                   [%s]
  UID                    [%s]
  riPid                  [%s]",

  $sMyName, $sPidFile, $sProc, $iUid, $riPid );

  #-----------------------------
  # Set return code to success..
  #-----------------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE; # Success..
  $$riPid = 0; # Reset..

  #-------------------------
  # Define local variables..
  #-------------------------
  my ( $iPid, $rhlProc, @lProcessHash ) = ();

  #----------------------------------
  # Start by testing the attributes..
  #----------------------------------
  if ( $sProc !~ /\S/ ) {
    &logError ( 20127, "<%s> Called with empty [%s] process string",
    $sMyName, $sProc );
    $iReturnCode = VCSDef_MonitorExit::OFFLINE; # Failure..
  }
  elsif ( ! -e $sPidFile ) {
    &logTrace ( 20128, "<%s> PID file [%s] does not exist",
    $sMyName, $sPidFile );
    $iReturnCode = VCSDef_MonitorExit::OFFLINE; # Failure..
  }
  elsif ( ! -s $sPidFile ) {
    &logWarn ( 20129, "<%s> PID file [%s] is of zero bytes",
    $sMyName, $sPidFile );
    $iReturnCode = VCSDef_MonitorExit::OFFLINE; # Failure..
  }

  #---------------------
  # Return, if offline..
  #---------------------
  return $iReturnCode unless VCSDef_MonitorExit::ONLINE == $iReturnCode;
 
  if ( open ( PIDFILE, $sPidFile ) ) {
    ( $iPid ) = grep { ( s/\n// || 1 ) && # Chomp..
                         s/.*?(\d+)$/$1/  # Get PID..
                } <PIDFILE>;

    $iPid = '' unless defined $iPid;      # Avoid undefs..

    close ( PIDFILE ) ||                  # Close file..
    &logWarn ( 20130,
    "<%s> Could not close PID file [%s] after read [%s]",
    $sMyName, $sPidFile, $! );
  }
  else {
    &logError ( 20131,
    "<%s> Failed to open PID file [%s] for read: [%s]",
    $sMyName, $sPidFile, $! );
    return VCSDef_MonitorExit::OFFLINE; # Failure..
  }

  #------------------------
  # Check if PID is valid..
  #------------------------
  if ( $iPid =~ /\d+/ &&     # If the valid number..
       kill ( 0, $iPid ) ) { # is a PID that's alive..

    &logTrace ( 20132,
    "<%s> PID [%s] from PID file [%s] exists",
    $sMyName, $iPid, $sPidFile );

    #-----------------------
    # Search in proc table..
    #-----------------------
    if ( SAPComponents::GetPidDetails ( $iPid,
                                        \@lProcessHash ) ) {
      #--------------
      # Got details..
      #--------------
      foreach $rhlProc ( @lProcessHash ) {
        &logTrace ( 20133,
        "<%s> Process details: UID [%s] PID [%s] PPID [%s] ARGS [%s]",
        $sMyName, $$rhlProc { UID }, $$rhlProc { PID }, $$rhlProc { PPID },
        $$rhlProc { COMMAND } );

        #--------------
        # Now compare..
        #--------------
        if ( ( $$rhlProc { UID } == $iUid ) &&       # Owner match..
               $$rhlProc { COMMAND } =~ /$sProc/ ) { # Filter match..
          &logTrace ( 20134,
          "<%s> PID [%s] owned by UID [%s] with ARGS [%s] contains [%s]",
          $sMyName, $iPid, $$rhlProc { UID }, $$rhlProc { COMMAND }, $sProc );
          $$riPid = $iPid; # Store..
        }
        else {
          &logTrace ( 20135,
          "<%s> Ignoring PID [%s] owned by UID [%s] with ARGS [%s] with string [%s]",
          $sMyName, $iPid, $$rhlProc { UID }, $$rhlProc { COMMAND }, $sProc );
          $iReturnCode = VCSDef_MonitorExit::OFFLINE; # Failure..
        }
      }
    }
    else {
      &logTrace ( 20136,
      "<%s> Could not get details for PID [%s]",
      $sMyName, $iPid );
      $iReturnCode = VCSDef_MonitorExit::OFFLINE; # Failure..
    }
  }
  else {
    &logTrace ( 20137, "<%s> PID [$iPid] does not exist",
    $sMyName, $sPidFile );
    $iReturnCode = VCSDef_MonitorExit::OFFLINE; # Failure..
  }

  @lProcessHash = (); # Truncate..
  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : bool KillSAPComponentsProcesses ( IN String InstProfile
                                              IN String ComponentType,
                                              IN String SAPAdmin );

 Description  : Kills processes for this instance of SAPComponents
 
 Parameters :

 $sInstProfile            Instance profile file for the SAP Component Instance
 $sComponentType          String identifier that classifies and describes the SAP Component.
 $sSAPAdmin               The dedicated user for the SAP Components
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------
=cut

sub KillSAPComponentsProcesses {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sInstProfile,
       $sComponentType,
       $sSAPAdmin ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20138, "Subroutine <%s> called with:
  InstProfile              [%s]
  ComponentType            [%s]
  SAPAdmin                 [%s]",

  $sMyName, $sInstProfile, $sComponentType, $sSAPAdmin );
 
  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1; # Success..

  #-------------------------
  # Set up local variables..
  #-------------------------
  my ( @liGoners, @liUnDead ) = ();

  #---------------------------------------------
  # Get list of processes running for this Uid..
  #---------------------------------------------
  if ( ( caller 1 )[3] eq 'SAPComponents::Start' ) { # During online..
    @liGoners = @liRunningProcs;              # Get PIDs to kill..
  }
  else {
    SAPComponents::GetProcState ( \@liGoners,
                                  $sComponentType,
                                  $sSAPAdmin,
                                  $sInstProfile,
                                  0 ); # Set filtering choice..
  }

  #--------------------------
  # Sort in ascending order..
  #--------------------------
  @liGoners = sort { $a <=> $b } @liGoners;

  #------------------------------------------------------------------
  # We do a two level kill. The SIGINQ ensures  that  the  PID  is  a
  # valid process before we actually issue the SIGKILL. We don't want
  # to report a failure for SIGKILL in case  of  defunct  or  missing
  # processes.
  #------------------------------------------------------------------
  if ( scalar @liGoners ) {
    #----------------------
    # Failed to kill some..
    #----------------------
    if ( @liUnDead = grep {
                         kill ( 0, $_ ) && # Valid..
                       ! kill ( 9, $_ )    # and could NOT be killed..
                     } @liGoners ) {       # From what's found..
      &logError ( 20139,
      "<%s> PIDs not killed [%s] Total not killed [%s]",
      $sMyName, join ( ' ', @liUnDead ), scalar @liUnDead );
      $iReturnCode = 0; # Failure..
    }
    else { # All killed..
      &logInfo ( 20140,
      "<%s> Killed PIDs [%s] Total killed: [%s]",
      $sMyName, join ( ' ', @liGoners ), scalar @liGoners );
    }
  }
  else {
    &logTrace ( 20141,
    "<%s> Could not locate any process this instance of SAPComponents",
    $sMyName );
  }

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : bool ExecLocalCmd ( IN String User,
                                IN String EnvFile,
                                IN String Command,
                                IN String CommandArgs,
                                OUT RefInt ExitCode,
                                IN Integer Timeout,
                                OUT ListReference Cmdout );

 Description : Wrapper function for
               Sys::RunWithEnvCmdWithOutputWithTimeOut to collect output
               of the command.
 
 Parameters :

 $sUser            UNIX user to run the command as
 $sEnvFile         Absolute path to location of EnvFile to source
 $sCommand         Absolute path to location of command
 $sCommandArgs     Arguments to command
 $riExitCode       Reference to collect exit code of command
 $iTimeout         Time limit to execute the command
 $rlsCmdout        Stores o/p from command if successful
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub ExecLocalCmd {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sUser,
       $sEnvFile,
       $sCommand,
       $sCommandArgs,
       $riExitCode,
       $iTimeout,
       $rlsCmdout ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20142, "Subroutine <%s> called with:
  User        [%s]
  EnvFile     [%s]
  Command     [%s]
  CommandArgs [%s]
  riExitCode  [%s]
  Timeout     [%s]
  rlsCmdout   [%s]",

  $sMyName,
  $sUser, $sEnvFile, $sCommand, $sCommandArgs, $riExitCode,
  $iTimeout, $rlsCmdout );

  #---------------------------------------
  # Setup return code to hopeful success..
  #---------------------------------------
  my $iReturnCode = 1;

  

  #-------------------
  # Set up arguments..
  #-------------------
  my $bSourceUserEnv = 0; # Do not source..
  my $bTimedOut = 0;      # Timeout variable..

  @$rlsCmdout = (); # Reset..

  #-----------
  # Validate..
  #-----------
  if ( ! -e $sCommand ) {
    &logError ( 20143,
    "<%s> Cmd [%s] does not exist", $sMyName, $sCommand );
    $iReturnCode = 0; # Failure..
  }
  elsif ( ! -x $sCommand ) {
    &logError ( 20144,
    "<%s> Cmd [%s] is not executable", $sMyName, $sCommand );
    $iReturnCode = 0; # Failure..
  }
  elsif ( $iTimeout <= 1 ) {
    &logError ( 20145,
    "<%s> Insufficient time [%s] to execute Cmd [%s] with Args [%s].",
    $sMyName, $iTimeout, $sCommand, $sCommandArgs );
    $iReturnCode = 0; # Failure..
  }

  $iReturnCode || return $iReturnCode; # Return if failed.. 

  #----------------------
  # Set LOGNAME to user..
  #----------------------
  my $sOldLogName = $ENV { LOGNAME }; # Backup..
  $ENV { LOGNAME } = $sUser;          # Now set..

  &logTrace ( 20146,
  "<%s> Executing Command [%s] with Args [%s]",
  $sMyName, $sCommand, $sCommandArgs );

  $iReturnCode = Sys::RunWithEnvCmdWithOutputWithTimeOut ( $sEnvFile,
                                                           $bSourceUserEnv,
                                                           $sCommand,
                                                           $sCommandArgs,
                                                           $riExitCode,
                                                           $sUser,
                                                           $rlsCmdout,
                                                           \$bTimedOut,
                                                           $iTimeout );

  #---------------------------
  # Remove trailing newlines..
  #---------------------------
  chomp @$rlsCmdout;

  #--------------------
  # Check return code..
  #--------------------
  if ( $iReturnCode ) {   # No error with command execution..
    if ( $$riExitCode ) { # Failure..
      &logTrace ( 20147,
      "<%s> Command [%s] with args [%s] returned exit code [%s]",
      $sMyName, $sCommand, $sCommandArgs, $$riExitCode );
      $iReturnCode = 0; # Failure..
    }
    else {
      &logTrace ( 20148,
      "<%s> Command [%s] with args [%s] executed successfully with exit code [%s]",
      $sMyName, $sCommand, $sCommandArgs, $$riExitCode );
    }
  }
  else { # Execution error..
    @$rlsCmdout = (); # Reset..

    if ( $bTimedOut ) {  # We timed out..
      &logError ( 20149,
      "<%s> Command [%s] with args [%s] did not complete within [%s] seconds",
      $sMyName, $sCommand, $sCommandArgs, $iTimeout );
      $$rlsCmdout [0] = 'timeout'; # Timeout..
    }
    else { # Failure due to something else..
      &logError ( 20150,
      "<%s> Failed in call to RunWithEnvCmdWithOutputWithTimeOut",
      $sMyName );
    }
  }

  #----------
  # Restore..
  #----------
  defined $sOldLogName ? $ENV { LOGNAME } = $sOldLogName : delete $ENV { LOGNAME };

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : bool GetPidDetails ( IN Integer Pid,
                                 OUT RefList ProcList );


 Description : Gets long process details for a known Pid if running

 Parameters :

 $iPid         PID whose details are requested
 $rlProcList   Ref to List to store details
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub GetPidDetails {
  #----------------
  # Get Arguments..
  #----------------
  my ( $iPid,
       $rlProcList ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20151, "Subroutine <%s> called with:
  Pid             [%s]
  rlProcList      [%s]",

  $sMyName, $iPid, $rlProcList );

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1;

  #---------------
  # Validate PID..
  #---------------
  unless ( defined $iPid    &&   # Catch undefs..
           $iPid =~ /^\d+$/ &&   # And invalid PIDs..
           kill ( 0, $iPid ) ) { # And defunct ones..
    &logError ( 20152,
    "<%s> PID [%s] is invalid", $sMyName, $iPid );
    return 0;
  }

  #-------------------------
  # Define local variables..
  #-------------------------
  my ( $sPsCmd, $sPsCmdArgs, $sLang, $sFmt, $sUnix95, $sSwitch, $iExitCode,
       %hsProc ) = ();

  $sSwitch = $^O;    # Get OS..
  @$rlProcList = (); # Truncate..

  #-------------
  # Store LANG..
  #-------------
  $sLang = $ENV { LANG };
  $ENV { LANG } = 'C';

  #---------------------------
  # Switch to the correct OS..
  #---------------------------
	if("solaris" eq $sSwitch) {

      my $sOSVer = ( uname ) [2]; # Get OS version..
      if ($sOSVer =~ /\d+\.(\d+)/) {
	$sOSVer = $1;
      }
      $sPsCmd = "/usr/bin/ps";
      if ( $sOSVer < 11 ) {
        $sPsCmd = "/usr/ucb/ps" if ( -x "/usr/ucb/ps" );
      }
      $sPsCmdArgs = "axwwl $iPid";
      $sFmt = '[\da-z]+\s+([\d]+)\s+([\d]+)\s+([\d]+)\s+.*?[\d]:[\d]{2}\s(.*)';

  }

	elsif("hpux" eq $sSwitch) {

      #-----------------------
      # Force XPG4 behaviour..
      #-----------------------
      $sUnix95 = $ENV { UNIX95 };
      $ENV { UNIX95 } = 'XPG4';

      $sPsCmd = "/usr/bin/ps";
      $sPsCmdArgs = "-p $iPid -xxlo uid,pid,ppid,args";
      $sFmt = '([\d]+)\s+([\d]+)\s+([\d]+)\s+(.*)';

  }

	elsif("aix" eq $sSwitch) {

      $sPsCmd = "/usr/bin/ps";
      $sPsCmdArgs = "axwwl $iPid";
      $sFmt =  '[\da-e]+\s+[A-Z]\s+([\d]+)\s+([\d]+)\s+([\d]+)\s+.*?[\d]:[\d]{2}\s(.*)';

	}


	elsif("linux" eq $sSwitch) {
     
      $sPsCmd = "/bin/ps";
      $sPsCmdArgs = "wwo uid,pid,ppid,args $iPid";
      $sFmt = '([\d]+)\s+([\d]+)\s+([\d]+)\s+(.*)';

	}

  &logTrace ( 20153, "<%s> OS [%s] PsCmd [%s] PsCmdArgs [%s]",
  $sMyName, $sSwitch, $sPsCmd, $sPsCmdArgs );

  #-------------------
  # Validate command..
  #-------------------
  if ( ! -x $sPsCmd ) {
    &logError ( 20154,
    "<%s> ps(1b) command [%s] is not executable", $sMyName, $sPsCmd );
    return 0; # Failure..
  }

  #-------------------------------
  # Fire command and get details..
  #-------------------------------
  @hsProc { qw ( UID PID PPID COMMAND ) } =
  map {
    s/\n//;                             # Chomp..
    s/^\s+//;                           # Strip leading spaces..
    /^$sFmt$/o;                         # Extract fields..
  } readpipe ( "$sPsCmd $sPsCmdArgs" ); # From the command..

  $iExitCode = $? >> 8; # Get exit code..

  #------------------------
  # Restore env variables..
  #------------------------
  defined $sLang ? $ENV { LANG } = $sLang : delete $ENV { LANG };
  defined $sUnix95 ? $ENV { UNIX95 } = $sUnix95 : delete $ENV { UNIX95 };

  #------------------
  # Check exit code..
  #------------------
  if ( $iExitCode ) {
    &logWarn ( 20155,
    "<%s> Cmd [%s] with args [%s] failed with exit code [%s]",
    $sMyName, $sPsCmd, $sPsCmdArgs, $iExitCode );

    my $sResourceName = $ENV { VCS_LOG_RESOURCE_NAME };
    &logWarn ( 20330,
    "<%s> Deleting PID files for VCS resource [%s]",
    $sMyName, $sResourceName );

    SAPComponents::DeletePidFile ( $sResourceName );

    %hsProc = (); # Truncate..
    return 0;
  }

  #---------------------
  # Fill up the bucket..
  #---------------------
  push ( @$rlProcList, \%hsProc ); # Update..

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : bool GetSrvrDetails ( IN String ComponentType,
                                  IN String InstProfile,
                                  OUT RefHashList SrvrDetails );

 Description : Gives PID file and process string for given server type

 Parameters  :

 $sComponentType            String identifier that classifies and describes the SAP Component.
 $sInstProfile              Specifies the full path and file name of the SAP Instance Profile.
 $rhlSrvrDetails            Stores details of PID file and process string
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub GetSrvrDetails {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sComponentType,
       $sInstProfile,
       $rhlSrvrDetails ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20156, "Subroutine <%s> called with:
  ComponentType            [%s]
  InstProfile              [%s]
  SrvrDetails              [%s]",

  $sMyName, $sComponentType, $sInstProfile, $rhlSrvrDetails );

  #-----------------------------
  # Set return code to success..
  #-----------------------------
  my $iReturnCode = 1; # Set to true..

  #-------------------
  # Define variables..
  #-------------------
  my ( $sKey, $sProc, $sPidFile, $sPIDDir, $sResourceType,
       $sResourceName, $iPat, $sSAPHome, $sSAPSID, $sInstName, 
       $sSAPHost, $iInstID, $sTmpProc, @lsWords, @lsProcMon) = ();

  #----------------------------------------------
  # Derive our resource type and resource names..
  #----------------------------------------------
  $sResourceType = ( $0 =~ m@([^/]+)@g ) [-2];
  $sResourceName   = $ENV { VCS_LOG_RESOURCE_NAME };
  $sPIDDir = "/tmp/.VRTS$sResourceType/$sResourceName/PID";

  #------------------------------
  # Create PID folder if needed..
  #------------------------------
  if ( -d $sPIDDir ) {
    &logTrace ( 20157, "<%s> PID folder [%s] exists",
    $sMyName, $sPIDDir );
  }
  elsif ( ! mkpath ( $sPIDDir, 0, 0700 ) ) {
    &logWarn ( 20158, "<%s> Failed to create PID folder [%s]",
    $sMyName, $sPIDDir );
    return 0; # Cannot proceed..
  }
  
  ( @lsWords ) = split( /\//, $sInstProfile );
  ( $sSAPSID, $sInstName, $sSAPHost ) = split ( /_/, $lsWords [-1] );
  ( $iInstID ) = $sInstName =~ /(\d{2})$/o;
  $sSAPHome = "/usr/sap/$sSAPSID/$sInstName/exe";
  
  #---------------------------------------
  # Define ProcMon for different Component
  #---------------------------------------
  push @lsProcMon, 'sapstart';
  if (  $sComponentType =~ /^Gateway$/o ){
        push @lsProcMon, 'gw';
  }
  elsif (  $sComponentType =~ /^WebDispatcher$/o ) {
        push @lsProcMon, 'wd';
  }
  elsif (  $sComponentType =~ /^ContentServerWD$/o ) {
        push @lsProcMon, 'cs';
  }
  elsif (  $sComponentType =~ /^MDM$/o ){
	push @lsProcMon, 'sapstartsrv';
	push @lsProcMon, 'md';
	push @lsProcMon, 'mdexe';
  }
  
  %$rhlSrvrDetails = (); # Truncate..

  #-----------------------------------------------
  # Set appropriate filter strings and PID files..
  #-----------------------------------------------
  
   foreach $sProc ( @lsProcMon ) {
    $sKey = "srchstr_${sProc}";
    $sPidFile = "$sPIDDir/pidfile.$sKey";

    if ( ( $sProc eq 'sapstart') || ( $sProc eq 'sapstartsrv')  ) {
      $sTmpProc = qr|$sProc\spf=.*$sSAPSID.*$sInstName.*$sSAPHost|;
    }
    elsif ( ( $sProc eq 'wd' ) || ( $sProc eq 'gw' ) || ( $sProc eq 'cs' )  ) {
      $sTmpProc = qr|$sProc.sap${sSAPSID}_${sInstName}.*pf=.*profile/${sSAPSID}_${sInstName}_${sSAPHost}|;
    }
    elsif (  $sProc eq 'md' ) {
      $sTmpProc = qr|.*${sProc}[is]*s.*sap${sSAPSID}_${sInstName}|;
    }
    elsif (  $sProc eq 'mdexe' ) {
      $sTmpProc = qr|.*${sSAPSID}/${sInstName}/exe/md[is]*s.*|;
    }

    @{ $$rhlSrvrDetails { $sProc } } = ( $sPidFile, $sTmpProc ); # Store..

    &logTrace ( 20159, "<%s> Generating filter [%s] for process [%s] with PID file [%s]",
    $sMyName, $sTmpProc, $sProc, $sPidFile );
  }


  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : int GetEngineTimeout ( IN Bool UseMargin );

 Description : Calculates engine timeout for entry point
 
 Parameters  :

 $bUseMargin      Flag to indicate if safety margin is requested

                  boolean value in the set of {0,1} where
                  0 No safety margin needed
                  1 Safety margin to be used
 Returns:  Time in secs.
 -------------------------------------------------------------------------

=cut

sub GetEngineTimeout {
  #----------------
  # Get Arguments..
  #----------------
  my ( $bUseMargin ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20160, "Subroutine <%s> called with:
  UseMargin  [%s]",
  $sMyName, $bUseMargin );

  #-------------------------
  # Define local variables..
  #-------------------------
  my ( $sAttribute, $sCommand, $sCommandArgs,
       $iEngineTimeout, $iExitCode ) = ();

  #----------------------------------
  # Grace margins from VCS timeouts..
  #----------------------------------
  my %hiGraceMargin = ( 'MonitorTimeout', 10,   # Seconds..
                        'OnlineTimeout',  10,
                        'OfflineTimeout', 10,
                        'CleanTimeout',   20 );

  #--------------------------------------------------
  # Attempt to get resource level overridden values..
  #--------------------------------------------------
  my $sResourceName = $ENV { VCS_LOG_RESOURCE_NAME };
  my ( $sEntryPointName, $sResourceType ) = reverse $0 =~ m@([^/]+)@g;
  ( $sAttribute = $sEntryPointName ) =~ s/(.*)/\u$1Timeout/;

  #-----------------------------------
  # Query if it has already been set..
  #-----------------------------------
  if ( defined $::giEPTimeout &&    # Defined..
       $::giEPTimeout =~ /^\d+/ ) { # to an integer..
    $iEngineTimeout = $::giEPTimeout;

    &logTrace ( 20161,
    "<%s> Using AEPTimeout [%s] as EP timeout value",
    $sMyName, $iEngineTimeout );
  }
  else {
    #-----------------------------------------
    # Form query string based on entry point..
    #-----------------------------------------
    $sCommand = $sCmdHares; # Get global..
    $sCommandArgs = "-value $sResourceName $sAttribute";
  
    #----------------------------------
    # Fire command and collect output..
    #----------------------------------
    ( $iEngineTimeout = readpipe ( "$sCommand $sCommandArgs 2>/dev/null" ) )
                      =~ s/\n//;
  
    #--------------------
    # Check exit status..
    #--------------------
    if ( $iExitCode = $? >> 8 ) {
      &logTrace ( 20162,
      "<%s> Command [%s] with Args [%s] failed with exitcode [%s]",
      $sMyName, $sCommand, $sCommandArgs, $iExitCode );
    }
    elsif ( ! $iEngineTimeout ) { # Undefined value..
      &logTrace ( 20163, "<%s> Could not get [%s] from engine",
      $sMyName, $sAttribute );
    }
    else { # Success..
      &logTrace ( 20164, "<%s> Got [%s] as [%s] sec from engine",
      $sMyName, $sAttribute, $iEngineTimeout );
    }
  
    #-----------------------------------------
    # Form query string based on entry point..
    #-----------------------------------------
    unless ( $iEngineTimeout ) {
      $sCommand = $sCmdHatype; # Get global..
      ( $sAttribute = $sEntryPointName ) =~ s/(.*)/\u$1Timeout/;
      $sResourceType = "SAPComponents" if $sResourceType !~ /\w/; # Ensure..
      $sCommandArgs = "-value $sResourceType $sAttribute";
    
      #----------------------------------
      # Fire command and collect output..
      #----------------------------------
      ( $iEngineTimeout = readpipe ( "$sCommand $sCommandArgs" ) ) =~ s/\n//;
    
      #--------------------
      # Check exit status..
      #--------------------
      if ( $iExitCode = $? >> 8 ) {
        &logTrace ( 20165,
        "<%s> Command [%s] with Args [%s] failed with exitcode [%s]",
        $sMyName, $sCommand, $sCommandArgs, $iExitCode );
        $iEngineTimeout = $bUseMargin ? $hiGraceMargin { $sAttribute } : 0;
      }
      elsif ( ! $iEngineTimeout ) { # Empty value..
        &logTrace ( 20166, "<%s> Could not get [%s] from engine",
        $sMyName, $sAttribute );
        $iEngineTimeout = $bUseMargin ? $hiGraceMargin { $sAttribute } : 0;
      }
      else { # Success..
        &logTrace ( 20167, "<%s> Got [%s] as [%s] sec from engine",
        $sMyName, $sAttribute, $iEngineTimeout );
      }
    }
  }

  #----------------------------
  # Apply margin if requested.. 
  #----------------------------
  if ( $bUseMargin     &&                                    # Margin requested..
       $iEngineTimeout >= $hiGraceMargin { $sAttribute } ) { # And valid..
    $iEngineTimeout -= $hiGraceMargin { $sAttribute };       # Apply..

    &logTrace ( 20168,
    "<%s> Setting up [%s] sec margin. Returning [%s]",
    $sMyName, $hiGraceMargin { $sAttribute }, $iEngineTimeout );
  }

  return ( $iEngineTimeout ); # Secs..
}


=pod

-------------------------------------------------------------------------
Function : bool DeletePidFile ( IN String ResourceName );


Description : Deletes PID files for the VCS resource.

Parameters :

$sResourceName ResourceName for which the PID files needs to be deleted.

Returns : boolean value in the set of {0,1} where
           0 false
           1 true
-------------------------------------------------------------------------

=cut

sub DeletePidFile {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sResourceName ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20329, "Subroutine <%s> called with:
  ResourceName        [%s]",

  $sMyName, $sResourceName );

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1;

  my ( $sPidFileDir, $iRet ) = ();
  #-------------------
  # Delete PID files..
  #-------------------
  &logTrace ( 20344,
  "<%s> Deleting PID files for VCS resource [%s]",
  $sMyName, $sResourceName );

  $sPidFileDir = "/tmp/.VRTSSAPComponents/" . $sResourceName . "/PID" ;

  if ( -d $sPidFileDir ) {
    $iRet = system("/bin/rm -rf $sPidFileDir") ;

    if ( $iRet == 0 ) {
      &logTrace ( 20331,
      "<%s> Successfully deleted PID files in the directory [%s].",
      $sMyName, $sPidFileDir );
      $iReturnCode = 0;
    }
    else {
      &logWarn ( 20332,
      "<%s> Failed to delete PID files in directory [%s]. Remove the files manually.",
      $sMyName, $sPidFileDir );
      $iReturnCode = 1;
    }
  }
  else {
    &logTrace ( 20333,
    "<%s> PID files for VCS resource [%s] doesn't exist.",
    $sMyName, $sResourceName );
  }

  return $iReturnCode;
}


1; # So PERL knows its included..
